#!/usr/bin/env python3
"""AutoSceneGeneratorエンティティのユニットテスト
TDD RED段階:失敗するテストを先に作成


仕様書: SPEC-DOMAIN-ENTITIES
"""

from unittest.mock import Mock

import pytest

pytestmark = pytest.mark.plot_episode

from noveler.domain.entities.auto_scene_generator import AutoSceneGenerator
from noveler.domain.exceptions import BusinessRuleViolationError
from noveler.domain.value_objects.generation_options import GenerationOptions
from noveler.domain.value_objects.project_context import ProjectContext
from noveler.domain.value_objects.scene_template import SceneTemplate


class TestAutoSceneGenerator:
    """AutoSceneGeneratorエンティティのテスト"""

    @pytest.mark.spec("SPEC-AUTO_SCENE_GENERATOR-CREATE_AUTO_SCENE_GE")
    def test_create_auto_scene_generator_with_valid_context(self) -> None:
        """有効なプロジェクトコンテキストでジェネレーターを作成"""
        # Arrange
        project_context = ProjectContext(
            project_name="test_project", genre="ファンタジー", protagonist_name="勇者", total_episodes=50
        )

        # Act
        generator = AutoSceneGenerator()
        generator.set_project_context(project_context)

        # Assert
        assert generator.project_context == project_context
        assert generator.project_context.project_name == "test_project"
        assert generator.project_context.genre == "ファンタジー"

    @pytest.mark.spec("SPEC-AUTO_SCENE_GENERATOR-GENERATE_SCENE_WITH_")
    def test_generate_scene_with_climax_template(self) -> None:
        """クライマックステンプレートでシーン生成"""
        # Arrange
        project_context = ProjectContext(
            project_name="test_project",
            genre="ファンタジー",
            protagonist_name="勇者",
            main_characters=[{"name": "勇者", "role": "主人公"}, {"name": "魔王", "role": "アンタゴニスト"}],
        )

        options = GenerationOptions(title="最終決戦", importance_level="S", detail_level="full")

        # Act
        generator = AutoSceneGenerator()
        generator.set_project_context(project_context)

        # まずクライマックステンプレートを追加
        climax_template = SceneTemplate.create_default_template("climax_scenes")
        generator.add_template(climax_template)

        result = generator.generate_scene("climax_scenes", "final_battle", options)

        # Assert
        assert result.category == "climax_scenes"
        assert result.scene_id == "final_battle"
        assert result.title == "最終決戦"
        assert result.importance_level == "S"
        assert result.auto_generated is True

    @pytest.mark.spec("SPEC-AUTO_SCENE_GENERATOR-VALIDATE_GENERATION_")
    def test_validate_generation_rules_with_insufficient_context(self) -> None:
        """不十分なコンテキストでルール検証"""
        # Arrange
        minimal_context = ProjectContext(
            project_name="test",
            genre="テスト",  # 有効なジャンル
        )

        options = GenerationOptions(importance_level="S")

        # Act & Assert - 最小限のコンテキストでも生成可能(デフォルトテンプレートがあるため)
        generator = AutoSceneGenerator(project_context=minimal_context)
        result = generator.generate_scene("climax_scenes", "test_scene", options)

        # 生成は成功するが、情報が限定的
        assert result.category == "climax_scenes"
        assert result.scene_id == "test_scene"
        assert result.auto_generated is True

    @pytest.mark.spec("SPEC-AUTO_SCENE_GENERATOR-GET_GENERATION_HISTO")
    def test_get_generation_history_empty_initially(self) -> None:
        """初期状態での生成履歴は空"""
        # Arrange
        context = ProjectContext(project_name="test", genre="ファンタジー")
        generator = AutoSceneGenerator(project_context=context)

        # Act
        history = generator.get_generation_history()

        # Assert
        assert history == []

    @pytest.mark.spec("SPEC-AUTO_SCENE_GENERATOR-ADD_CUSTOM_TEMPLATE_")
    def test_add_custom_template_and_retrieve(self) -> None:
        """カスタムテンプレートの追加と取得"""
        # Arrange
        context = ProjectContext(project_name="test", genre="ファンタジー")
        custom_template = SceneTemplate(
            name="custom_test", category="custom_scenes", setting_patterns={"location": ["test_location"]}
        )

        # Act
        generator = AutoSceneGenerator(project_context=context)
        generator.add_custom_template(custom_template)
        retrieved = generator.get_template("custom_test")

        # Assert
        assert retrieved is not None
        assert retrieved.name == "custom_test"

    @pytest.mark.spec("SPEC-AUTO_SCENE_GENERATOR-GENERATE_MULTIPLE_SC")
    def test_generate_multiple_scenes_updates_history(self) -> None:
        """複数シーン生成で履歴が更新される"""
        # Arrange
        context = ProjectContext(project_name="test", genre="ファンタジー")
        options1 = GenerationOptions(title="Scene 1")
        options2 = GenerationOptions(title="Scene 2")

        # Act
        generator = AutoSceneGenerator(project_context=context)

        generator.generate_scene("emotional_scenes", "scene1", options1)
        generator.generate_scene("romance_scenes", "scene2", options2)

        history = generator.get_generation_history()

        # Assert
        assert len(history) == 2
        assert history[0]["scene_id"] == "scene1"
        assert history[1]["scene_id"] == "scene2"

    @pytest.mark.spec("SPEC-AUTO_SCENE_GENERATOR-VALIDATE_SCENE_ID_UN")
    def test_validate_scene_id_uniqueness(self) -> None:
        """シーンID重複の検証"""
        # Arrange
        context = ProjectContext(project_name="test", genre="ファンタジー")
        options = GenerationOptions(title="Test Scene")

        # Act & Assert
        generator = AutoSceneGenerator(project_context=context)

        # 最初の生成は成功
        generator.generate_scene("emotional_scenes", "duplicate_id", options)

        # 同じIDでの生成はエラーになる想定
        with pytest.raises(BusinessRuleViolationError, match=".*"):
            generator.generate_scene("romance_scenes", "duplicate_id", options)

    @pytest.mark.spec("SPEC-AUTO_SCENE_GENERATOR-GET_SCENE_STATISTICS")
    def test_get_scene_statistics_with_generated_scenes(self) -> None:
        """生成済みシーンの統計取得"""
        # Arrange
        context = ProjectContext(project_name="test", genre="ファンタジー")

        # Act & Assert
        generator = AutoSceneGenerator(project_context=context)

        # 複数のシーンを生成
        for i in range(3):
            options = GenerationOptions(title=f"Scene {i}")
            generator.generate_scene("emotional_scenes", f"scene_{i}", options)

        stats = generator.get_scene_statistics()
        assert stats["total_scenes"] == 3
        assert stats["by_category"]["emotional_scenes"] == 3

    @pytest.mark.spec("SPEC-AUTO_SCENE_GENERATOR-CLEAR_GENERATION_HIS")
    def test_clear_generation_history(self) -> None:
        """生成履歴のクリア"""
        # Arrange
        context = ProjectContext(project_name="test", genre="ファンタジー")

        # Act & Assert
        generator = AutoSceneGenerator(project_context=context)

        # シーンを生成
        options = GenerationOptions(title="Test")
        generator.generate_scene("emotional_scenes", "test", options)

        # 履歴をクリア
        generator.clear_generation_history()

        history = generator.get_generation_history()
        assert len(history) == 0

    @pytest.mark.spec("SPEC-AUTO_SCENE_GENERATOR-PROJECT_CONTEXT_UPDA")
    def test_project_context_update_affects_generation(self) -> None:
        """プロジェクトコンテキスト更新による生成結果の変化"""
        # Arrange
        original_context = ProjectContext(project_name="test", genre="ファンタジー")
        updated_context = ProjectContext(project_name="test", genre="恋愛")

        # Act & Assert
        generator = AutoSceneGenerator(project_context=original_context)

        # コンテキスト更新
        generator.update_project_context(updated_context)

        # 更新後のコンテキストが反映されているか確認
        assert generator.project_context.genre == "恋愛"


class TestAutoSceneGeneratorWithMockRepository:
    """モックリポジトリを使用したテスト"""

    @pytest.mark.spec("SPEC-AUTO_SCENE_GENERATOR-GENERATE_SCENE_WITH_")
    def test_generate_scene_with_repository_integration(self) -> None:
        """リポジトリ統合での生成(将来の拡張を想定したテスト)"""
        # Arrange
        Mock()
        context = ProjectContext(project_name="test", genre="ファンタジー")

        # Act & Assert - 現在の実装ではリポジトリは直接使用しない
        generator = AutoSceneGenerator(project_context=context)

        options = GenerationOptions(title="Test Scene")
        result = generator.generate_scene("climax_scenes", "test", options)

        # 生成結果を確認
        assert result.category == "climax_scenes"
        assert result.scene_id == "test"
        assert result.title == "Test Scene"

    @pytest.mark.spec("SPEC-AUTO_SCENE_GENERATOR-ERROR_HANDLING_WHEN_")
    def test_error_handling_when_repository_fails(self) -> None:
        """リポジトリエラー時のハンドリング(将来の拡張を想定)"""
        # Arrange
        mock_repo = Mock()
        mock_repo.load_project_files.side_effect = Exception("Repository error")

        context = ProjectContext(project_name="test", genre="ファンタジー")

        # Act & Assert - 現在の実装ではリポジトリエラーは発生しない
        generator = AutoSceneGenerator(project_context=context)

        options = GenerationOptions(title="Test")

        # 正常に生成される(リポジトリに依存しないため)
        result = generator.generate_scene("climax_scenes", "test", options)
        assert result.scene_id == "test"


class TestAutoSceneGeneratorBusinessRules:
    """ビジネスルールのテスト"""

    @pytest.mark.spec("SPEC-AUTO_SCENE_GENERATOR-MINIMUM_REQUIRED_PRO")
    def test_minimum_required_project_info_validation(self) -> None:
        """最小限の必要プロジェクト情報の検証"""
        # Arrange - 最小限の情報のみ
        minimal_context = ProjectContext(project_name="minimal", genre="テスト")

        # Act & Assert
        generator = AutoSceneGenerator(project_context=minimal_context)

        # 最小限の情報でも基本的な生成は可能である想定
        options = GenerationOptions(title="Minimal Scene")
        result = generator.generate_scene("emotional_scenes", "minimal", options)

        assert result.title == "Minimal Scene"

    @pytest.mark.spec("SPEC-AUTO_SCENE_GENERATOR-OPTIMAL_PROJECT_INFO")
    def test_optimal_project_info_enhances_generation(self) -> None:
        """充実したプロジェクト情報による生成品質向上"""
        # Arrange - 充実した情報
        rich_context = ProjectContext(
            project_name="rich_project",
            genre="ファンタジー",
            protagonist_name="勇者リック",
            main_characters=[
                {"name": "勇者リック", "role": "主人公"},
                {"name": "魔法使いエルフィ", "role": "ヒロイン"},
                {"name": "魔王ダーク", "role": "アンタゴニスト"},
            ],
            total_episodes=100,
        )

        # Act & Assert
        generator = AutoSceneGenerator(project_context=rich_context)

        options = GenerationOptions(title="Epic Battle", importance_level="S", detail_level="full")

        result = generator.generate_scene("climax_scenes", "epic", options)

        # 充実した情報により詳細なシーンが生成される想定
        assert len(result.characters) >= 2  # 主人公とアンタゴニスト
        assert result.importance_level == "S"

    @pytest.mark.spec("SPEC-AUTO_SCENE_GENERATOR-SCENE_CATEGORY_VALID")
    def test_scene_category_validation(self) -> None:
        """シーンカテゴリの検証"""
        # Arrange
        context = ProjectContext(project_name="test", genre="ファンタジー")

        # Act & Assert
        generator = AutoSceneGenerator(project_context=context)

        options = GenerationOptions(title="Test")

        # 無効なカテゴリでエラーになる想定
        with pytest.raises(BusinessRuleViolationError, match=".*"):
            generator.generate_scene("invalid_category", "test", options)

    @pytest.mark.spec("SPEC-AUTO_SCENE_GENERATOR-IMPORTANCE_LEVEL_AFF")
    def test_importance_level_affects_generation_detail(self) -> None:
        """重要度レベルによる生成詳細度の変化"""
        # Arrange
        context = ProjectContext(project_name="test", genre="ファンタジー")

        # Act & Assert
        generator = AutoSceneGenerator(project_context=context)

        # S級の重要シーン
        high_options = GenerationOptions(title="Critical Scene", importance_level="S", detail_level="full")

        # C級の普通シーン
        low_options = GenerationOptions(title="Minor Scene", importance_level="C", detail_level="basic")

        high_scene = generator.generate_scene("climax_scenes", "high", high_options)
        low_scene = generator.generate_scene("emotional_scenes", "low", low_options)

        # 両方のシーンが生成されることを確認
        assert high_scene.importance_level == "S"
        assert low_scene.importance_level == "C"

        # 詳細レベルの違いが反映されていることを確認
        # (現在の実装では完成度スコアは同じになる可能性があるため、
        # 重要度レベルの違いを確認)
        assert high_scene.get_completion_score() >= low_scene.get_completion_score()



"""Infrastructure.git.hooks.plot_version_post_commit
Where: Infrastructure module defining the plot version post-commit hook.
What: Generates plot version updates after commits.
Why: Keeps plot version data in sync with repository changes.
"""

from noveler.presentation.shared.shared_utilities import console

"プロットコミット後の自動バージョン提案スクリプト\nGit post-commit hook から呼び出される\n"
import subprocess
import sys
from pathlib import Path
from typing import Any

project_root = Path(__file__).parent.parent
sys.path.insert(0, str(project_root))
try:
    from noveler.application.use_cases.bidirectional_interactive_confirmation import (
        BidirectionalInteractiveConfirmation,
    )
    from noveler.application.use_cases.chapter_consistency_interactive_confirmation import (
        ChapterConsistencyInteractiveConfirmation,
    )
    from noveler.application.use_cases.interactive_consistency_confirmation import InteractiveConsistencyConfirmation
    from noveler.application.use_cases.interactive_version_proposal import InteractiveVersionProposal
    from noveler.application.use_cases.plot_commit_version_proposer import PlotCommitVersionProposer
    from noveler.domain.versioning.entities import (
        BidirectionalForeshadowingAnalyzer,
        ChapterPlotImpactAnalyzer,
        ForeshadowingStatusUpdater,
        PlotCommitAnalyzer,
        PlotVersionConsistencyAnalyzer,
    )
except ImportError as e:
    console.print(f"Import error: {e}")
    sys.exit(0)


class GitHookVersionProposer:
    """Git hook用のバージョン提案処理"""

    def __init__(self, project_path: Path, logger_service=None, console_service=None) -> None:
        self.project_path = project_path
        self.analyzer = PlotCommitAnalyzer()
        self.git_service = MockGitService(project_path)
        self.version_repo = MockVersionRepo()
        self.proposer = PlotCommitVersionProposer(self.analyzer, self.version_repo, self.git_service)
        self.interactive = InteractiveVersionProposal()
        self.consistency_analyzer = PlotVersionConsistencyAnalyzer()
        self.consistency_confirmation = InteractiveConsistencyConfirmation()
        self.chapter_analyzer = ChapterPlotImpactAnalyzer()
        self.chapter_confirmation = ChapterConsistencyInteractiveConfirmation()
        self.bidirectional_analyzer = BidirectionalForeshadowingAnalyzer()
        self.status_updater = ForeshadowingStatusUpdater()
        self.bidirectional_confirmation = BidirectionalInteractiveConfirmation()
        self.logger_service = logger_service
        self.console_service = console_service

    def run(self) -> None:
        """メイン処理を実行"""
        try:
            proposal = self.proposer.propose_version_update()
            if not proposal.should_propose:
                self.console_service.print("✅ プロット変更なし")
                return
            self.console_service.print("\n🔄 プロットコミット検出")
            self.console_service.print(f"変更ファイル: {', '.join(proposal.changed_files)}")
            result = self.interactive.handle_proposal(
                proposal.suggested_version, proposal.change_type, proposal.reason, self._input_handler
            )
            if result.accepted:
                self.console_service.print(f"✅ {result.message}")
                version_info = self._create_version(result.version, result.description)
                if version_info:
                    if proposal.change_type == "major":
                        self._handle_consistency_updates(version_info)
                    elif proposal.change_type == "minor":
                        self._handle_chapter_consistency_updates(version_info, proposal.changed_files)
            else:
                self.console_service.print(f"⏭️  {result.message}")
        except Exception as e:
            self.console_service.print(f"Warning: Git hook error: {e}")

    def _input_handler(self, prompt: str) -> str:
        """ユーザー入力を処理"""
        try:
            return input(prompt)
        except (KeyboardInterrupt, EOFError):
            return "n"

    def _create_version(self, version: str, description: str) -> dict[str, Any]:
        """実際のバージョン作成(簡略版)"""
        self.console_service.print(f"🏷️  プロットバージョン {version} を作成中...")
        self.console_service.print(f"📝 変更内容: {description}")
        return {
            "version": version,
            "description": description,
            "type": "major" if version.count(".") == 2 and version.split(".")[0] != version.split(".")[1] else "minor",
            "affected_chapters": [3, 4, 5],
        }

    def _handle_consistency_updates(self, version_info: dict) -> None:
        """整合性更新の確認・実行"""
        try:
            self.console_service.print("\n🔄 関連ファイルの整合性チェック中...")
            impact = self.consistency_analyzer.analyze_consistency_impact(version_info)
            impact_summary = {
                "affected_episodes": len(list(range(15, 25))),
                "affected_foreshadowing": 2,
                "requires_review": impact.affected_management_files,
            }
            self.console_service.print("📊 影響範囲:")
            self.console_service.print(f"   - 影響ファイル: {', '.join(impact.affected_management_files)}")
            self.console_service.print(f"   - 影響話数: {impact_summary['affected_episodes']}話")
            confirmation = self.consistency_confirmation.confirm_consistency_updates(
                impact_summary, self._input_handler
            )
            if confirmation.approved:
                self.console_service.print("🔧 整合性更新を実行中...")
                self.console_service.print("✅ 整合性更新完了")
            else:
                self.console_service.print("⏭️  整合性更新をスキップ")
        except Exception as e:
            self.console_service.print(f"Warning: {e}")
            self.console_service.print("💡 手動で関連ファイルの確認をお願いします")

    def _handle_chapter_consistency_updates(self, _version_info: dict, changed_files: list[str]) -> None:
        """章別プロット整合性更新の確認・実行(双方向伏線管理含む)"""
        try:
            self.console_service.print("\n🔄 章別プロットの整合性チェック中...")
            chapter_files = [f for f in changed_files if "章別プロット" in f]
            if len(chapter_files) == 1:
                impact = self.chapter_analyzer.analyze_chapter_impact(chapter_files[0])
                affected_chapter = impact.affected_chapter
                foreshadowing_data: dict[str, Any] = self._load_mock_foreshadowing_data()
                bidirectional_impact = self.bidirectional_analyzer.analyze_bidirectional_impact(
                    foreshadowing_data, affected_chapter
                )
                reverse_check_chapters = self.bidirectional_analyzer.get_reverse_check_chapters(bidirectional_impact)
                chapter_impact = {
                    "affected_chapter": affected_chapter,
                    "chapter_name": impact.chapter_name,
                    "affected_episodes": 3,
                    "affected_foreshadowing": len(
                        bidirectional_impact.setup_modified + bidirectional_impact.resolution_modified
                    ),
                }
                self.console_service.print(f"📊 {impact.chapter_name}プロットの変更を検出")
                if bidirectional_impact.has_bidirectional_impact:
                    self.console_service.print(f"🔄 {bidirectional_impact.impact_summary}")
                    bidirectional_data: dict[str, Any] = {
                        "affected_chapter": affected_chapter,
                        "setup_modified_count": len(bidirectional_impact.setup_modified),
                        "resolution_modified_count": len(bidirectional_impact.resolution_modified),
                        "reverse_check_chapters": list(reverse_check_chapters),
                        "impact_summary": bidirectional_impact.impact_summary,
                    }
                    bi_confirmation = self.bidirectional_confirmation.confirm_bidirectional_updates(
                        bidirectional_data, self._input_handler
                    )
                    if bi_confirmation.approved:
                        self.console_service.print("🔧 伏線ステータスを更新中...")
                        self.console_service.print(f"✅ {bi_confirmation.message}")
                        if bi_confirmation.include_reverse_check and reverse_check_chapters:
                            console.print(
                                f"\n💡 次のステップ: {', '.join(f'第{ch}章' for ch in reverse_check_chapters)}の確認を推奨"
                            )
                    else:
                        self.console_service.print(f"⏭️  {bi_confirmation.message}")
                else:
                    confirmation = self.chapter_confirmation.confirm_chapter_updates(
                        chapter_impact, self._input_handler
                    )
                    if confirmation.approved:
                        self.console_service.print(f"🔧 {impact.chapter_name}の整合性更新を実行中...")
                        self.console_service.print(f"✅ {confirmation.message}")
                    else:
                        self.console_service.print(f"⏭️  {confirmation.message}")
            elif len(chapter_files) > 1:
                impact = self.chapter_analyzer.analyze_multiple_chapters_impact(chapter_files)
                chapters_impact = {
                    "affected_chapters": impact.affected_chapters,
                    "total_affected_episodes": len(impact.affected_chapters) * 3,
                    "total_affected_foreshadowing": len(impact.affected_chapters),
                }
                self.console_service.print(
                    f"📊 複数章プロットの変更を検出: {', '.join(f'第{ch}章' for ch in impact.affected_chapters)}"
                )
                confirmation = self.chapter_confirmation.confirm_multiple_chapters_updates(
                    chapters_impact, self._input_handler
                )
                if confirmation.approved:
                    self.console_service.print("🔧 複数章の整合性更新を実行中...")
                    self.console_service.print(f"✅ {confirmation.message}")
                else:
                    self.console_service.print(f"⏭️  {confirmation.message}")
        except Exception as e:
            self.console_service.print(f"Warning: {e}")
            self.console_service.print("💡 手動で章別ファイルの確認をお願いします")

    def _load_mock_foreshadowing_data(self) -> dict:
        """モック伏線データ(TODO: 実際のファイル読み込みに置き換え)"""
        return {
            "foreshadow_001": {
                "description": "主人公の秘密",
                "target_chapter": 3,
                "resolution_chapter": 7,
                "status": "ACTIVE",
            },
            "foreshadow_002": {
                "description": "謎のアイテム",
                "target_chapter": 3,
                "resolution_chapter": 9,
                "status": "ACTIVE",
            },
        }


class MockGitService:
    """Git操作のモック(最小実装)"""

    def __init__(self, project_path: Path, logger_service=None, console_service=None) -> None:
        self.project_path = project_path
        self.logger_service = logger_service
        self.console_service = console_service

    def get_changed_files(self) -> list:
        """最新コミットの変更ファイルを取得"""
        try:
            result = subprocess.run(
                ["git", "diff", "--name-only", "HEAD~1", "HEAD"],
                cwd=self.project_path,
                capture_output=True,
                text=True,
                check=True,
            )
            return [f for f in result.stdout.strip().split("\n") if f]
        except subprocess.CalledProcessError:
            return []


class MockVersionRepo:
    """バージョンリポジトリのモック"""

    def get_current_version(self) -> str:
        """現在のバージョンを取得(ダミー)"""
        return "v1.0.0"


def main() -> None:
    """メインエントリーポイント"""
    try:
        result = subprocess.run(["git", "rev-parse", "--show-toplevel"], capture_output=True, text=True, check=True)
        Path(result.stdout.strip())
    except subprocess.CalledProcessError:
        console.print("Error: Cannot find project root")
        sys.exit(1)
    current_dir = Path.cwd()
    project_dir = None
    for parent in [current_dir, *list(current_dir.parents)]:
        if (parent / "プロジェクト設定.yaml").exists():
            project_dir = parent
            break
    if not project_dir:
        console.print("📝 小説プロジェクト外での変更のため、プロットバージョン管理をスキップします")
        sys.exit(0)
    proposer = GitHookVersionProposer(project_dir)
    proposer.run()


if __name__ == "__main__":
    main()
PlotVersionPostCommitHook = GitHookVersionProposer



"""Infrastructure.services.codemap_cache_service
Where: Infrastructure service handling codemap caching.
What: Stores and invalidates codemap data in cache to optimise performance.
Why: Improves codemap access speed across the application.
"""

from noveler.presentation.shared.shared_utilities import console

"CODEMAPキャッシュサービス\n\nCODEMAP_dependencies.yamlの高速読み込みとキャッシュ管理を提供。\n大規模ファイル（134KB+）のパフォーマンス問題を解決。\n\n設計原則:\n    - ファイルハッシュベースの変更検出\n    - Pickleによる高速バイナリキャッシュ\n    - 部分読み込みサポート\n    - メモリ効率的なデータ構造\n"
import hashlib
import json
import os
import pickle
import time
from dataclasses import dataclass, field
from datetime import datetime, timedelta, timezone
from pathlib import Path
from typing import Any

import yaml

from noveler.infrastructure.logging.unified_logger import get_logger


@dataclass
class CodeMapCacheEntry:
    """CODEMAPキャッシュエントリ"""

    file_path: str
    file_hash: str
    last_modified: datetime
    data: dict[str, Any]
    section: str
    load_time: float

    def is_valid(self, current_hash: str) -> bool:
        """キャッシュの有効性を検証"""
        return self.file_hash == current_hash


@dataclass
class CodeMapIndex:
    """高速検索用インデックス"""

    module_locations: dict[str, int] = field(default_factory=dict)
    layer_modules: dict[str, set[str]] = field(default_factory=dict)
    high_coupling_modules: set[str] = field(default_factory=set)
    violation_modules: set[str] = field(default_factory=set)
    updated_at: datetime = field(default_factory=datetime.now)


class CodeMapCacheService:
    """CODEMAPキャッシュ管理サービス

    責務:
        - CODEMAP_dependencies.yamlの高速読み込み
        - 分割ファイル構造のサポート
        - インデックスベースの選択的読み込み
        - キャッシュの自動管理と無効化
    """

    CACHE_VERSION = "1.0.0"
    CACHE_DIR = ".codemap_cache"
    INDEX_FILE = "codemap.index.json"
    MAX_CACHE_AGE = timedelta(hours=24)

    def __init__(self, project_root: Path) -> None:
        self.project_root = project_root
        self.cache_dir = project_root / self.CACHE_DIR
        self.cache_dir.mkdir(exist_ok=True)
        self.logger = get_logger(__name__)
        self._cache: dict[str, CodeMapCacheEntry] = {}
        self._index: CodeMapIndex | None = None

    def load_dependencies(
        self, section: str = "full", use_cache: bool = True, force_reload: bool = False
    ) -> dict[str, Any]:
        """CODEMAP依存関係を読み込む

        Args:
            section: 読み込むセクション ('core', 'violations', 'stats', 'full')
            use_cache: キャッシュを使用するか
            force_reload: 強制的に再読み込みするか

        Returns:
            要求されたセクションのデータ
        """
        start_time = time.time()
        file_path = self._get_file_path(section)
        if use_cache and (not force_reload):
            cached_data: dict[str, Any] = self._load_from_cache(file_path, section)
            if cached_data:
                load_time = time.time() - start_time
                console.print(f"Loaded {section} from cache in {load_time:.3f}s")
                return cached_data
        data = self._load_from_file(file_path)
        if use_cache:
            self._save_to_cache(file_path, section, data)
        load_time = time.time() - start_time
        console.print(f"Loaded {section} from file in {load_time:.3f}s")
        return data

    def load_module_dependencies(self, module_name: str) -> dict[str, Any]:
        """特定モジュールの依存関係のみを読み込む

        Args:
            module_name: モジュール名（例: 'noveler.application.use_cases.xxx'）

        Returns:
            モジュールの依存関係情報
        """
        if not self._index:
            self._build_index()
        full_data: dict[str, Any] = self.load_dependencies(section="core")
        core_deps = full_data.get("core_dependencies", {})
        if module_name in core_deps:
            return {
                "module": module_name,
                "imports": core_deps[module_name].get("imports", []),
                "imported_by": core_deps[module_name].get("imported_by", []),
            }
        return {"module": module_name, "imports": [], "imported_by": []}

    def get_layer_modules(self, layer: str) -> list[str]:
        """特定レイヤーのモジュール一覧を取得

        Args:
            layer: レイヤー名 ('domain', 'application', 'infrastructure', 'presentation')

        Returns:
            モジュール名のリスト
        """
        if not self._index:
            self._build_index()
        return list(self._index.layer_modules.get(layer, set()))

    def get_violations(self) -> list[dict[str, str]]:
        """アーキテクチャ違反情報を取得"""
        data = self.load_dependencies(section="violations")
        return data.get("dependency_issues", {}).get("layer_violations", [])

    def get_statistics(self) -> dict[str, Any]:
        """統計情報を取得"""
        return self.load_dependencies(section="stats")

    def clear_cache(self) -> None:
        """キャッシュをクリア"""
        self._cache.clear()
        for cache_file in self.cache_dir.glob("*.pickle"):
            cache_file.unlink()
        console.print("Cache cleared")

    def _get_file_path(self, section: str) -> Path:
        """セクションに対応するファイルパスを取得"""
        base_path = self.project_root
        if section == "full":
            return base_path / "CODEMAP_dependencies.yaml"
        if section == "core":
            core_file = base_path / "CODEMAP_dependencies_core.yaml"
            if core_file.exists():
                return core_file
            return base_path / "CODEMAP_dependencies.yaml"
        if section == "violations":
            violations_file = base_path / "CODEMAP_dependencies_violations.yaml"
            if violations_file.exists():
                return violations_file
            return base_path / "CODEMAP_dependencies.yaml"
        if section == "stats":
            stats_file = base_path / "CODEMAP_dependencies_stats.yaml"
            if stats_file.exists():
                return stats_file
            return base_path / "CODEMAP_dependencies.yaml"
        return base_path / "CODEMAP_dependencies.yaml"

    def _calculate_file_hash(self, file_path: Path) -> str:
        """ファイルのハッシュ値を計算"""
        hasher = hashlib.sha256()
        with file_path.open("rb") as f:
            for chunk in iter(lambda: f.read(4096), b""):
                hasher.update(chunk)
        return hasher.hexdigest()

    def _load_from_cache(self, file_path: Path, section: str) -> dict[str, Any] | None:
        """キャッシュから読み込み"""
        cache_key = f"{file_path.name}_{section}"
        if cache_key in self._cache:
            entry = self._cache[cache_key]
            current_hash = self._calculate_file_hash(file_path)
            if entry.is_valid(current_hash):
                return entry.data
        cache_file = self.cache_dir / f"{cache_key}.pickle"
        if cache_file.exists():
            try:
                with cache_file.open("rb") as f:
                    entry = pickle.load(f)
                current_hash = self._calculate_file_hash(file_path)
                if entry.is_valid(current_hash):
                    self._cache[cache_key] = entry
                    return entry.data
            except Exception as e:
                console.print(f"Failed to load cache: {e}")
        return None

    def _load_from_file(self, file_path: Path) -> dict[str, Any]:
        """ファイルから読み込み"""
        with file_path.open(encoding="utf-8") as f:
            return yaml.safe_load(f)

    def _save_to_cache(self, file_path: Path, section: str, data: dict[str, Any]) -> None:
        """キャッシュに保存"""
        cache_key = f"{file_path.name}_{section}"
        file_hash = self._calculate_file_hash(file_path)
        entry = CodeMapCacheEntry(
            file_path=str(file_path),
            file_hash=file_hash,
            last_modified=datetime.now(timezone.utc),
            data=data,
            section=section,
            load_time=0.0,
        )
        self._cache[cache_key] = entry
        cache_file = self.cache_dir / f"{cache_key}.pickle"
        try:
            with cache_file.open("wb") as f:
                pickle.dump(entry, f)
        except Exception as e:
            console.print(f"Failed to save cache: {e}")

    def _build_index(self) -> None:
        """インデックスを構築"""
        index_file = self.cache_dir / self.INDEX_FILE
        if index_file.exists():
            try:
                with index_file.open() as f:
                    index_data: dict[str, Any] = json.load(f)
                self._index = CodeMapIndex(
                    module_locations=index_data.get("module_locations", {}),
                    layer_modules={k: set(v) for (k, v) in index_data.get("layer_modules", {}).items()},
                    high_coupling_modules=set(index_data.get("high_coupling_modules", [])),
                    violation_modules=set(index_data.get("violation_modules", [])),
                    updated_at=datetime.fromisoformat(index_data.get("updated_at")),
                )
                return
            except Exception as e:
                console.print(f"Failed to load index: {e}")
        self._index = CodeMapIndex()
        data = self.load_dependencies(section="full", use_cache=False)
        core_deps = data.get("dependency_map", {}).get("core_dependencies", {})
        for module_name in core_deps:
            layer = self._get_layer(module_name)
            if layer:
                if layer not in self._index.layer_modules:
                    self._index.layer_modules[layer] = set()
                self._index.layer_modules[layer].add(module_name)
        violations: Any = data.get("dependency_map", {}).get("dependency_issues", {}).get("layer_violations", [])
        for violation in violations:
            self._index.violation_modules.add(violation.get("from", ""))
        self._save_index()

    def _get_layer(self, module_name: str) -> str | None:
        """モジュール名からレイヤーを判定"""
        if "domain" in module_name:
            return "domain"
        if "application" in module_name:
            return "application"
        if "infrastructure" in module_name:
            return "infrastructure"
        if "presentation" in module_name:
            return "presentation"
        return None

    def _save_index(self) -> None:
        """インデックスを保存"""
        if not self._index:
            return
        index_file = self.cache_dir / self.INDEX_FILE
        index_data: dict[str, Any] = {
            "module_locations": self._index.module_locations,
            "layer_modules": {k: list(v) for (k, v) in self._index.layer_modules.items()},
            "high_coupling_modules": list(self._index.high_coupling_modules),
            "violation_modules": list(self._index.violation_modules),
            "updated_at": self._index.updated_at.isoformat(),
        }
        try:
            with index_file.open("w") as f:
                json.dump(index_data, f, indent=2)
        except Exception as e:
            console.print(f"Failed to save index: {e}")


def get_codemap_cache_service() -> CodeMapCacheService:
    """CODEMAPキャッシュサービスのインスタンスを取得"""
    try:
        from noveler.presentation.shared.shared_utilities import get_test_path_service

        path_service = get_test_path_service()
        project_root = path_service.project_root
    except ImportError:
        project_root = Path(os.environ.get("PROJECT_ROOT", Path.cwd()))
    return CodeMapCacheService(project_root)



#!/usr/bin/env python3

"""Domain.entities.unified_analysis_context
Where: Domain entity describing context used in unified analysis.
What: Aggregates contextual data fed into analysis engines.
Why: Ensures analyses operate on consistent context information.
"""

from __future__ import annotations

"""統合分析コンテキストエンティティ

統合コンテキスト分析で使用する全ての情報を保持・管理するドメインエンティティ。
全A31項目（68項目）の完全コンテキスト保持と段階間関係性維持を実現。
"""


import hashlib
from collections.abc import Mapping
from dataclasses import dataclass, field
from typing import TYPE_CHECKING, Any

from noveler.domain.value_objects.analysis_scope import AnalysisScope

if TYPE_CHECKING:
    from pathlib import Path


@dataclass
class A31ChecklistItem:
    """A31チェックリスト項目"""

    id: str
    item: str
    phase: str
    required: bool
    item_type: str
    reference_guides: list[str] = field(default_factory=list)
    input_files: list[str] = field(default_factory=list)
    output_files: list[str] = field(default_factory=list)
    output_validation: list[str] = field(default_factory=list)
    status: bool = False
    auto_fix_supported: bool = False
    auto_fix_level: str | None = None

    def get_priority_score(self) -> float:
        """優先度スコアの計算

        Returns:
            float: 優先度スコア（高いほど重要）
        """
        base_score = 10.0 if self.required else 5.0

        # Phase別重み付け
        phase_weights = {
            "Phase2_執筆段階": 3.0,
            "Phase3_推敲段階": 2.5,
            "Phase4_品質チェック段階": 2.0,
            "Phase1_設計・下書き段階": 1.5,
            "Phase5_完成処理段階": 1.0,
            "Phase0_自動執筆開始": 1.0,
            "公開前最終確認": 1.2,
        }

        phase_weight = phase_weights.get(self.phase, 1.0)
        return base_score * phase_weight

    def has_cross_references(self) -> bool:
        """相互参照の有無

        Returns:
            bool: 相互参照があるかどうか
        """
        return len(self.input_files) > 0 or len(self.output_files) > 0


@dataclass
class ProjectContext:
    """プロジェクトコンテキスト"""

    project_name: str
    project_root: Path
    episode_number: int
    manuscript_path: Path
    character_settings: dict[str, Any] = field(default_factory=dict)
    plot_context: dict[str, Any] = field(default_factory=dict)
    world_settings: dict[str, Any] = field(default_factory=dict)
    previous_episodes: list[dict[str, Any]] = field(default_factory=list)

    def get_context_fingerprint(self) -> str:
        """コンテキストフィンガープリントの生成

        Returns:
            str: ユニークなコンテキスト識別子
        """
        context_data: dict[str, Any] = (
            f"{self.project_name}_{self.episode_number}_{len(self.character_settings)}_{len(self.plot_context)}"
        )
        return hashlib.md5(context_data.encode()).hexdigest()[:12]


@dataclass
class CrossReferenceData:
    """相互参照データ"""

    relationship_count: int
    file_dependencies: dict[str, list[str]] = field(default_factory=dict)
    content_relationships: dict[str, list[str]] = field(default_factory=dict)
    phase_interactions: dict[str, list[str]] = field(default_factory=dict)

    def get_relationship_density(self) -> float:
        """関係密度の計算

        Returns:
            float: 関係密度（0.0-1.0）
        """
        if self.relationship_count == 0:
            return 0.0

        total_possible = 68 * (68 - 1) // 2  # 68項目の完全グラフ
        return min(self.relationship_count / total_possible, 1.0)


@dataclass
class UnifiedAnalysisContext:
    """統合分析コンテキストエンティティ

    統合コンテキスト分析で使用する全ての情報を統合管理。
    全A31項目（68項目）の完全コンテキスト保持と関係性維持を実現。
    """

    manuscript_content: str
    a31_checklist: dict[str, list[A31ChecklistItem]] | Any
    project_context: ProjectContext
    episode_context: dict[str, Any]
    cross_reference_data: CrossReferenceData
    preservation_scope: AnalysisScope

    # メタデータ
    context_version: str = "1.0"
    creation_timestamp: str | None = None
    content_hash: str | None = None

    def __post_init__(self) -> None:
        """初期化後処理"""
        if not isinstance(self.a31_checklist, Mapping):
            self.a31_checklist = {}

        if self.content_hash is None:
            self.content_hash = self._calculate_content_hash()

    # ---- checklist helpers -------------------------------------------------
    def _iter_checklist_groups(self) -> list[list[A31ChecklistItem]]:
        """安全にチェックリスト群をイテレートする。

        Mock が渡された場合やリスト以外の値が含まれる場合でも
        可能な範囲で変換し、問題がある項目は無視する。
        """

        if not getattr(self, "a31_checklist", None):
            return []

        try:
            values = self.a31_checklist.values()
        except AttributeError:
            return []

        try:
            iterable_values = list(values)
        except TypeError:
            return []

        groups: list[list[A31ChecklistItem]] = []
        for items in iterable_values:
            if isinstance(items, list):
                groups.append(items)
                continue
            try:
                converted = list(items)
            except TypeError:
                continue
            groups.append(converted)  # type: ignore[arg-type]
        return groups

    @classmethod
    def from_a31_yaml(
        cls,
        a31_yaml_content: dict[str, Any],
        manuscript_content: str,
        project_context: ProjectContext,
        preservation_scope: AnalysisScope = AnalysisScope.COMPREHENSIVE,
    ) -> UnifiedAnalysisContext:
        """A31 YAMLからコンテキスト生成

        Args:
            a31_yaml_content: A31チェックリストYAML内容
            manuscript_content: 原稿内容
            project_context: プロジェクトコンテキスト
            preservation_scope: 保持範囲

        Returns:
            UnifiedAnalysisContext: 統合分析コンテキスト
        """
        # A31項目のパース
        checklist_items = {}
        checklist_raw = a31_yaml_content.get("checklist_items", {})

        for phase_name, items in checklist_raw.items():
            if phase_name in {"metadata", "validation_summary"}:
                continue

            parsed_items = [
                A31ChecklistItem(
                    id=item_data.get("id", ""),
                    item=item_data.get("item", ""),
                    phase=phase_name,
                    required=item_data.get("required", False),
                    item_type=item_data.get("type", "unknown"),
                    reference_guides=item_data.get("reference_guides", []),
                    input_files=item_data.get("input_files", []),
                    output_files=item_data.get("output_files", []),
                    output_validation=item_data.get("output_validation", []),
                    status=item_data.get("status", False),
                    auto_fix_supported=item_data.get("auto_fix_supported", False),
                    auto_fix_level=item_data.get("auto_fix_level"),
                )
                for item_data in items
                if isinstance(item_data, dict) and "id" in item_data
            ]

            if parsed_items:
                checklist_items[phase_name] = parsed_items

        # 相互参照データの構築
        relationship_count = 0
        file_dependencies = {}
        content_relationships = {}
        phase_interactions = {}

        for phase_name, items in checklist_items.items():
            phase_interactions[phase_name] = []
            for item in items:
                if item.has_cross_references():
                    relationship_count += 1
                    file_dependencies[item.id] = item.input_files + item.output_files

                    # 他項目との関係性分析
                    for other_phase_name, other_items in checklist_items.items():
                        if other_phase_name != phase_name:
                            for other_item in other_items:
                                if any(ref in other_item.input_files for ref in item.output_files):
                                    if item.id not in content_relationships:
                                        content_relationships[item.id] = []
                                    content_relationships[item.id].append(other_item.id)
                                    phase_interactions[phase_name].append(other_phase_name)

        cross_reference_data: dict[str, Any] = CrossReferenceData(
            relationship_count=relationship_count,
            file_dependencies=file_dependencies,
            content_relationships=content_relationships,
            phase_interactions=phase_interactions,
        )

        # エピソードコンテキストの構築
        episode_context = {
            "episode_number": project_context.episode_number,
            "manuscript_length": len(manuscript_content),
            "total_items": sum(len(items) for items in checklist_items.values()),
            "required_items": sum(1 for items in checklist_items.values() for item in items if item.required),
            "auto_fixable_items": sum(
                1 for items in checklist_items.values() for item in items if item.auto_fix_supported
            ),
        }

        return cls(
            manuscript_content=manuscript_content,
            a31_checklist=checklist_items,
            project_context=project_context,
            episode_context=episode_context,
            cross_reference_data=cross_reference_data,
            preservation_scope=preservation_scope,
        )

    def get_total_items_count(self) -> int:
        """総項目数の取得

        Returns:
            int: 総項目数
        """
        return sum(len(items) for items in self._iter_checklist_groups())

    def get_required_items_count(self) -> int:
        """必須項目数の取得

        Returns:
            int: 必須項目数
        """
        return sum(
            1 for items in self._iter_checklist_groups() for item in items if getattr(item, "required", False)
        )

    def get_priority_items(self, limit: int = 0) -> list[A31ChecklistItem]:
        """優先度順項目リストの取得

        Args:
            limit: 取得制限数（0は無制限）

        Returns:
            list[A31ChecklistItem]: 優先度順項目リスト
        """
        all_items: list[A31ChecklistItem] = []
        for items in self._iter_checklist_groups():
            all_items.extend(items)

        # 優先度でソート
        sorted_items = sorted(all_items, key=lambda x: x.get_priority_score(), reverse=True)

        if limit > 0:
            return sorted_items[:limit]
        return sorted_items

    def get_phase_items(self, phase_name: str) -> list[A31ChecklistItem]:
        """段階別項目の取得

        Args:
            phase_name: 段階名

        Returns:
            list[A31ChecklistItem]: 段階別項目リスト
        """
        if hasattr(self.a31_checklist, "get"):
            items = self.a31_checklist.get(phase_name, [])
            if isinstance(items, list):
                return items
            try:
                return list(items)
            except TypeError:
                return []
        return []

    def get_cross_phase_relationships(self) -> dict[str, list[str]]:
        """段階間関係の取得

        Returns:
            dict[str, list[str]]: 段階間関係マップ
        """
        return self.cross_reference_data.phase_interactions

    def should_preserve_full_context(self) -> bool:
        """完全コンテキスト保持判定

        Returns:
            bool: 完全コンテキスト保持を行うかどうか
        """
        return self.preservation_scope.includes_context_preservation()

    def get_analysis_item_limit(self) -> int:
        """分析項目制限の取得

        Returns:
            int: 分析項目数制限（0は無制限）
        """
        return self.preservation_scope.get_item_limit()

    def generate_context_summary(self) -> dict[str, Any]:
        """コンテキストサマリーの生成

        Returns:
            dict[str, Any]: コンテキストサマリー
        """
        return {
            "project_name": self.project_context.project_name,
            "episode_number": self.project_context.episode_number,
            "manuscript_length": len(self.manuscript_content),
            "total_items": self.get_total_items_count(),
            "required_items": self.get_required_items_count(),
            "phase_count": len(self._iter_checklist_groups()),
            "relationship_count": self.cross_reference_data.relationship_count,
            "relationship_density": self.cross_reference_data.get_relationship_density(),
            "preservation_scope": self.preservation_scope.value,
            "full_context_preservation": self.should_preserve_full_context(),
            "context_fingerprint": self.project_context.get_context_fingerprint(),
            "content_hash": self.content_hash,
        }

    def validate_context_integrity(self) -> dict[str, Any]:
        """コンテキスト整合性の検証

        Returns:
            dict[str, Any]: 検証結果
        """
        issues = []
        warnings = []

        # 基本検証
        if not self.manuscript_content:
            issues.append("原稿内容が空です")

        if not self._iter_checklist_groups():
            issues.append("A31チェックリストが空です")

        if self.get_total_items_count() != 68:
            warnings.append(f"項目数が期待値（68）と異なります: {self.get_total_items_count()}")

        # コンテキスト保持検証
        if self.should_preserve_full_context() and len(self.manuscript_content) < 1000:
            warnings.append("完全コンテキスト保持モードですが原稿が短すぎます")

        # 関係性検証
        if self.cross_reference_data.relationship_count == 0:
            warnings.append("項目間の関係性が検出されていません")

        return {
            "is_valid": len(issues) == 0,
            "issues": issues,
            "warnings": warnings,
            "validation_summary": {
                "manuscript_ok": bool(self.manuscript_content),
                "checklist_ok": bool(self._iter_checklist_groups()),
                "relationships_ok": self.cross_reference_data.relationship_count > 0,
                "context_preservation_ok": self.should_preserve_full_context() or len(self.manuscript_content) > 0,
            },
        }

    def _calculate_content_hash(self) -> str:
        """コンテンツハッシュの計算

        Returns:
            str: コンテンツハッシュ
        """
        content_data: dict[str, Any] = (
            f"{self.manuscript_content}_{self.get_total_items_count()}_{self.preservation_scope.value}"
        )
        return hashlib.sha256(content_data.encode()).hexdigest()[:16]

    def __str__(self) -> str:
        """文字列表現"""
        return f"UnifiedAnalysisContext(episode={self.project_context.episode_number}, items={self.get_total_items_count()}, scope={self.preservation_scope.value})"

    def __repr__(self) -> str:
        """開発者向け表現"""
        return f"UnifiedAnalysisContext(project='{self.project_context.project_name}', episode={self.project_context.episode_number}, total_items={self.get_total_items_count()}, preservation_scope={self.preservation_scope})"



# File: tests/integration/test_ten_stage_episode_writing_integration.py
# Purpose: Integration tests for TenStageEpisodeWritingUseCase with TenStageProgressUseCase
# Context: Phase 2-B validation - ensures proper integration between components

"""Integration tests for TenStageEpisodeWritingUseCase Phase 2-B integration."""

import asyncio
from unittest.mock import AsyncMock, MagicMock, patch

import pytest

from noveler.application.services.a30_compatibility_adapter import (
    A30CompatibilityAdapter,
    CompatibilityMode,
)
from noveler.application.use_cases.ten_stage_episode_writing_use_case import (
    TenStageEpisodeWritingUseCase,
)
from noveler.application.use_cases.ten_stage_progress_use_case import (
    TenStageProgressUseCase,
)
from noveler.domain.value_objects.detailed_execution_stage import DetailedExecutionStage
from noveler.domain.value_objects.five_stage_writing_execution import (
    ExecutionStage,
    FiveStageWritingRequest,
    FiveStageWritingResponse,
    StageExecutionResult,
    StageExecutionStatus,
)


@pytest.fixture
def project_root(tmp_path):
    """Create a temporary project root for testing."""
    import yaml

    project = tmp_path / "test_project"
    project.mkdir()
    (project / "manuscripts").mkdir()
    (project / "config").mkdir()

    # Create required plot file structure
    plot_dir = project / "20_プロット" / "話別プロット"
    plot_dir.mkdir(parents=True)

    # Create sample plot file for episode 1
    plot_file = plot_dir / "第001話_プロット.yaml"
    plot_data = {
        "summary": "これはテスト用の話別プロットです。",
        "beats": [
            "起: テストの導入",
            "承: テストの展開",
            "転: テストの山場",
            "結: テストの締め"
        ]
    }
    with plot_file.open("w", encoding="utf-8") as f:
        yaml.dump(plot_data, f, default_flow_style=False, allow_unicode=True)

    return project


@pytest.fixture
def writing_request(project_root):
    """Create a sample writing request."""
    return FiveStageWritingRequest(
        episode_number=1,
        project_root=project_root,
        genre="fantasy",
        viewpoint="三人称単元視点",
        viewpoint_character="主人公",
        word_count_target=3500
    )


@pytest.fixture
def progress_use_case():
    """Create TenStageProgressUseCase instance."""
    return TenStageProgressUseCase()


@pytest.fixture
def compatibility_adapter():
    """Create A30CompatibilityAdapter instance."""
    return A30CompatibilityAdapter(CompatibilityMode.A30_DETAILED_TEN_STAGE)


@pytest.fixture
def episode_writing_use_case(progress_use_case, compatibility_adapter):
    """Create TenStageEpisodeWritingUseCase with dependencies."""
    with patch("noveler.application.use_cases.ten_stage_episode_writing_use_case.ConfigurationLoaderService"):
        use_case = TenStageEpisodeWritingUseCase(
            progress_use_case=progress_use_case,
            compatibility_adapter=compatibility_adapter
        )

    stage_results = {
        ExecutionStage.DATA_COLLECTION: StageExecutionResult(
            stage=ExecutionStage.DATA_COLLECTION,
            status=StageExecutionStatus.COMPLETED,
            execution_time_ms=120.0,
            turns_used=2,
            output_data={"stage": "data_collection"},
        ),
        ExecutionStage.PLOT_ANALYSIS: StageExecutionResult(
            stage=ExecutionStage.PLOT_ANALYSIS,
            status=StageExecutionStatus.COMPLETED,
            execution_time_ms=140.0,
            turns_used=2,
            output_data={"stage": "plot_analysis"},
        ),
        ExecutionStage.EPISODE_DESIGN: StageExecutionResult(
            stage=ExecutionStage.EPISODE_DESIGN,
            status=StageExecutionStatus.COMPLETED,
            execution_time_ms=180.0,
            turns_used=3,
            output_data={"stage": "episode_design"},
        ),
        ExecutionStage.MANUSCRIPT_WRITING: StageExecutionResult(
            stage=ExecutionStage.MANUSCRIPT_WRITING,
            status=StageExecutionStatus.COMPLETED,
            execution_time_ms=220.0,
            turns_used=4,
            output_data={"stage": "manuscript_writing", "manuscript": "stub"},
        ),
        ExecutionStage.QUALITY_FINALIZATION: StageExecutionResult(
            stage=ExecutionStage.QUALITY_FINALIZATION,
            status=StageExecutionStatus.COMPLETED,
            execution_time_ms=160.0,
            turns_used=3,
            output_data={"stage": "quality_finalization"},
        ),
    }

    detailed_turns_total = sum(stage.expected_turns for stage in DetailedExecutionStage)

    stub_response = FiveStageWritingResponse(
        success=True,
        session_id="test_session",
        stage_results=stage_results,
        total_turns_used=detailed_turns_total,
        total_execution_time_ms=sum(result.execution_time_ms for result in stage_results.values()),
        total_cost_usd=0.8,
    )

    executor_mock = AsyncMock()
    executor_mock.execute_unified_session = AsyncMock(return_value=stub_response)
    use_case._session_executor = executor_mock
    use_case._check_claude_availability = AsyncMock(return_value=True)
    return use_case


class TestTenStageEpisodeWritingIntegration:
    """Integration tests for TenStageEpisodeWritingUseCase with Phase 2-B features."""

    @pytest.mark.asyncio
    async def test_ten_stage_workflow_with_progress_tracking(
        self, episode_writing_use_case, writing_request, progress_use_case
    ):
        """Test that 10-stage workflow properly integrates with progress tracking."""
        # Initialize progress tracking first
        from noveler.application.use_cases.ten_stage_progress_use_case import TenStageProgressRequest

        init_request = TenStageProgressRequest(
            episode_number=1,
            project_root=writing_request.project_root,
            operation="start"
        )
        await progress_use_case.execute(init_request)

        # Execute the 10-stage workflow
        response = await episode_writing_use_case.execute(writing_request)

        assert response.success is True
        assert response.session_id is not None
        assert response.stage_results is not None
        assert len(response.stage_results) > 0

        # Verify progress was tracked
        progress_query = TenStageProgressRequest(
            episode_number=1,
            project_root=writing_request.project_root,
            operation="query"
        )
        progress_response = await progress_use_case.execute(progress_query)

        assert progress_response.success is True
        assert progress_response.overall_progress > 0
        assert len(progress_response.completed_stages) > 0

    @pytest.mark.asyncio
    async def test_compatibility_adapter_stage_mapping(
        self, episode_writing_use_case, writing_request
    ):
        """Test that compatibility adapter properly maps between 5 and 10 stages."""
        adapter = episode_writing_use_case._compatibility_adapter

        # Test all detailed stages map to legacy stages
        for detailed_stage in DetailedExecutionStage:
            legacy_stage = adapter.convert_detailed_to_legacy(detailed_stage)
            assert isinstance(legacy_stage, ExecutionStage)

            # Test reverse mapping
            detailed_stages = adapter.convert_legacy_to_detailed(legacy_stage)
            assert detailed_stage in detailed_stages

    @pytest.mark.asyncio
    async def test_progress_tracking_initialization(
        self, episode_writing_use_case, writing_request, progress_use_case
    ):
        """Test that progress tracking is properly initialized at workflow start."""
        response = await episode_writing_use_case.execute(writing_request)

        assert response.success is True

        from noveler.application.use_cases.ten_stage_progress_use_case import TenStageProgressRequest

        progress_query = TenStageProgressRequest(
            episode_number=1,
            project_root=writing_request.project_root,
            operation="query"
        )
        progress_response = await progress_use_case.execute(progress_query)

        assert progress_response.success is True
        assert progress_response.stage_progresses is not None

        # All stages should be initialized
        assert len(progress_response.stage_progresses) == len(DetailedExecutionStage)

    @pytest.mark.asyncio
    async def test_stage_specific_prompt_generation(
        self, episode_writing_use_case, writing_request
    ):
        """Test that stage-specific prompts are properly generated for each stage."""
        adapter = episode_writing_use_case._compatibility_adapter

        context = {
            "episode_number": writing_request.episode_number,
            "genre": writing_request.genre,
            "viewpoint": writing_request.viewpoint,
        }

        for detailed_stage in DetailedExecutionStage:
            prompt_data = adapter.create_stage_specific_prompt(detailed_stage, context)

            assert prompt_data is not None
            assert "system" in prompt_data
            assert "user" in prompt_data
            # Check that stage info is in either system or user prompt
            # Stage information might be in Japanese, so check for key concepts
            combined = prompt_data["system"] + prompt_data["user"]
            # At minimum, the prompt should contain some information
            assert len(combined) > 0

    @pytest.mark.asyncio
    async def test_structured_output_creation(
        self, episode_writing_use_case, writing_request
    ):
        """Test that structured outputs are properly created for each stage."""
        adapter = episode_writing_use_case._compatibility_adapter

        for detailed_stage in DetailedExecutionStage:
            raw_output = {
                "stage": detailed_stage.value,
                "result": f"Test result for {detailed_stage.value}",
                "episode_number": writing_request.episode_number,
            }

            quality_scores = {
                "overall": 0.85,
                "completeness": 0.90,
                "consistency": 0.80,
            }

            structured_output = adapter.create_compatible_output(
                detailed_stage, raw_output, quality_scores
            )

            assert structured_output is not None
            assert structured_output.step_id == f"step_{detailed_stage.value.lower()}"
            assert structured_output.quality_metrics.overall_score == 0.85
            assert structured_output.structured_data == raw_output

    @pytest.mark.asyncio
    async def test_legacy_stage_result_aggregation(
        self, episode_writing_use_case, writing_request, progress_use_case
    ):
        """Test that detailed stage results are properly aggregated into legacy stages."""
        # Initialize progress tracking
        from noveler.application.use_cases.ten_stage_progress_use_case import TenStageProgressRequest

        init_request = TenStageProgressRequest(
            episode_number=1,
            project_root=writing_request.project_root,
            operation="start"
        )
        await progress_use_case.execute(init_request)

        # Execute workflow
        with patch.object(episode_writing_use_case, "_save_generated_manuscript") as mock_save:
            response = await episode_writing_use_case.execute(writing_request)

            assert response.success is True
            assert response.stage_results is not None

            # Verify that we have stage results
            # Note: Not all legacy stages may be present as some are aggregated
            assert len(response.stage_results) > 0

            # Verify key stages are present
            assert ExecutionStage.DATA_COLLECTION in response.stage_results
            assert ExecutionStage.MANUSCRIPT_WRITING in response.stage_results

            # Verify stage results have proper structure
            for stage_result in response.stage_results.values():
                assert stage_result.status is not None
                assert stage_result.turns_used > 0

    @pytest.mark.asyncio
    async def test_progress_query_after_execution(
        self, episode_writing_use_case, writing_request, progress_use_case
    ):
        """Test that progress can be queried after execution completes."""
        # Initialize progress tracking
        from noveler.application.use_cases.ten_stage_progress_use_case import TenStageProgressRequest

        init_request = TenStageProgressRequest(
            episode_number=1,
            project_root=writing_request.project_root,
            operation="start"
        )
        await progress_use_case.execute(init_request)

        # Execute workflow
        with patch.object(episode_writing_use_case, "_save_generated_manuscript") as mock_save:
            response = await episode_writing_use_case.execute(writing_request)

            assert response.success is True

            # Query final progress
            progress_query = TenStageProgressRequest(
                episode_number=1,
                project_root=writing_request.project_root,
                operation="query"
            )
            progress_response = await progress_use_case.execute(progress_query)

            assert progress_response.success is True
            assert progress_response.overall_progress == 1.0  # All stages completed
            assert len(progress_response.completed_stages) == len(DetailedExecutionStage)

    @pytest.mark.asyncio
    async def test_execution_metrics_tracking(
        self, episode_writing_use_case, writing_request
    ):
        """Test that execution metrics are properly tracked."""
        with patch.object(episode_writing_use_case, "_save_generated_manuscript") as mock_save:
            response = await episode_writing_use_case.execute(writing_request)

            assert response.success is True
            assert response.total_turns_used > 0
            assert response.total_cost_usd > 0
            assert response.total_execution_time_ms > 0

            # Verify turn counts match expectations
            expected_turns = sum(stage.expected_turns for stage in DetailedExecutionStage)
            assert response.total_turns_used == expected_turns

    @pytest.mark.asyncio
    async def test_manuscript_generation_and_saving(
        self, episode_writing_use_case, writing_request, project_root, progress_use_case
    ):
        """Test that manuscript is generated and saved properly."""
        # Initialize progress tracking
        from noveler.application.use_cases.ten_stage_progress_use_case import TenStageProgressRequest

        init_request = TenStageProgressRequest(
            episode_number=1,
            project_root=writing_request.project_root,
            operation="start"
        )
        await progress_use_case.execute(init_request)

        response = await episode_writing_use_case.execute(writing_request)

        assert response.success is True
        assert response.manuscript_path is not None

        # The file should exist if manuscript was actually saved
        # In test environment, this might be a temporary path
        if response.manuscript_path.exists():
            # Verify manuscript content
            manuscript_content = response.manuscript_path.read_text(encoding="utf-8")
            assert f"第{writing_request.episode_number:03d}話" in manuscript_content
        else:
            # At minimum, the path should be set
            assert str(response.manuscript_path).endswith(".md")

    @pytest.mark.asyncio
    async def test_error_handling_in_stage_execution(
        self, episode_writing_use_case, writing_request, progress_use_case
    ):
        """Test that errors in stage execution are properly handled."""
        # Force the unified executor to raise an error
        original_executor = episode_writing_use_case._session_executor.execute_unified_session
        episode_writing_use_case._session_executor.execute_unified_session.side_effect = RuntimeError("Simulated stage failure")
        try:
            response = await episode_writing_use_case.execute(writing_request)
        finally:
            episode_writing_use_case._session_executor.execute_unified_session = original_executor

        assert response.success is False
        assert response.error_message is not None
        assert "Simulated stage failure" in response.error_message

    @pytest.mark.asyncio
    async def test_progress_export_report(
        self, episode_writing_use_case, writing_request, progress_use_case
    ):
        """Test that progress report can be exported after execution."""
        # Initialize progress tracking
        from noveler.application.use_cases.ten_stage_progress_use_case import TenStageProgressRequest

        init_request = TenStageProgressRequest(
            episode_number=1,
            project_root=writing_request.project_root,
            operation="start"
        )
        await progress_use_case.execute(init_request)

        # Execute workflow
        with patch.object(episode_writing_use_case, "_save_generated_manuscript") as mock_save:
            response = await episode_writing_use_case.execute(writing_request)

            assert response.success is True

            # Export progress report
            report = progress_use_case.export_progress_report(1)

            assert report["episode"] == 1
            assert report["overall_progress"] == 1.0
            assert report["status"] == "completed"
            assert len(report["stages"]["completed"]) == len(DetailedExecutionStage)
            assert report["metrics"]["total_turns_used"] > 0

    @pytest.mark.asyncio
    async def test_compatibility_mode_configuration(
        self, progress_use_case, project_root
    ):
        """Test that different compatibility modes can be configured."""
        # Test with LEGACY_FIVE_STAGE mode
        legacy_adapter = A30CompatibilityAdapter(CompatibilityMode.LEGACY_FIVE_STAGE)

        with patch("noveler.application.use_cases.ten_stage_episode_writing_use_case.ConfigurationLoaderService") as mock_config:
            mock_config.return_value.load_project_settings.return_value = MagicMock()
            legacy_use_case = TenStageEpisodeWritingUseCase(
                progress_use_case=progress_use_case,
                compatibility_adapter=legacy_adapter
            )

            # Verify the adapter is using the correct mode
            # The mode is stored internally in the adapter
            assert isinstance(legacy_use_case._compatibility_adapter, A30CompatibilityAdapter)
            # Can check functionality instead of direct mode access
            execution_plan = legacy_use_case._compatibility_adapter.get_execution_plan(1)
            assert execution_plan["mode"] == "legacy_five_stage"

        # Test with HYBRID_GRADUAL_MIGRATION mode
        hybrid_adapter = A30CompatibilityAdapter(CompatibilityMode.HYBRID_GRADUAL_MIGRATION)

        with patch("noveler.application.use_cases.ten_stage_episode_writing_use_case.ConfigurationLoaderService") as mock_config:
            mock_config.return_value.load_project_settings.return_value = MagicMock()
            hybrid_use_case = TenStageEpisodeWritingUseCase(
                progress_use_case=progress_use_case,
                compatibility_adapter=hybrid_adapter
            )

            # Verify through execution plan
            assert isinstance(hybrid_use_case._compatibility_adapter, A30CompatibilityAdapter)
            execution_plan = hybrid_use_case._compatibility_adapter.get_execution_plan(1)
            assert execution_plan["mode"] == "hybrid_gradual_migration"

    @pytest.mark.asyncio
    async def test_stage_estimation_accuracy(
        self, episode_writing_use_case
    ):
        """Test that stage execution estimates are accurate."""
        estimates = episode_writing_use_case.get_stage_execution_estimates()

        expected_total = sum(s.expected_turns for s in DetailedExecutionStage)
        assert estimates["total_estimated_turns"] == expected_total
        assert len(estimates["stage_breakdown"]) == len(DetailedExecutionStage)

        # Verify each stage in breakdown
        for i, detailed_stage in enumerate(DetailedExecutionStage):
            stage_info = estimates["stage_breakdown"][i]
            assert stage_info["stage"] == detailed_stage.value
            assert stage_info["estimated_turns"] == detailed_stage.expected_turns

    @pytest.mark.asyncio
    async def test_concurrent_execution_prevention(
        self, episode_writing_use_case, writing_request, progress_use_case
    ):
        """Test that concurrent executions for the same episode are prevented."""
        # Start first execution
        task1 = asyncio.create_task(
            episode_writing_use_case.execute(writing_request)
        )

        # Try to start another execution for the same episode
        await asyncio.sleep(0.1)  # Small delay to ensure first starts

        # This should handle the concurrent execution gracefully
        task2 = asyncio.create_task(
            episode_writing_use_case.execute(writing_request)
        )

        # Wait for both to complete
        response1, response2 = await asyncio.gather(task1, task2, return_exceptions=True)

        # At least one should succeed
        successful = [r for r in [response1, response2] if not isinstance(r, Exception) and r.success]
        assert len(successful) >= 1



#!/usr/bin/env python3
"""WriterLevelServiceのユニットテスト

TDD原則に従い、執筆レベル判定サービスのビジネスロジックをテスト


仕様書: SPEC-UNIT-TEST
"""

import pytest

from noveler.domain.services.writer_level_service import WriterLevelService
from noveler.domain.value_objects.quality_standards import WriterLevel


class TestWriterLevelService:
    """WriterLevelServiceのテスト"""

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-DETERMINE_LEVEL_BEGI")
    def test_determine_level_beginner_with_few_episodes(self) -> None:
        """エピソード数が少ない場合は初心者レベルと判定される"""
        # Given
        completed_episodes = 3
        average_quality_score = 70.0

        # When
        level = WriterLevelService.determine_level(completed_episodes, average_quality_score)

        # Then
        assert level == WriterLevel.BEGINNER

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-DETERMINE_LEVEL_BEGI")
    def test_determine_level_beginner_boundary(self) -> None:
        """エピソード数5件は初心者レベルの境界値"""
        # Given
        completed_episodes = 5
        average_quality_score = 70.0

        # When
        level = WriterLevelService.determine_level(completed_episodes, average_quality_score)

        # Then
        assert level == WriterLevel.BEGINNER

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-DETERMINE_LEVEL_INTE")
    def test_determine_level_intermediate_with_moderate_episodes(self) -> None:
        """エピソード数が6-20の場合は中級者レベルと判定される"""
        # Given
        completed_episodes = 10
        average_quality_score = 70.0

        # When
        level = WriterLevelService.determine_level(completed_episodes, average_quality_score)

        # Then
        assert level == WriterLevel.INTERMEDIATE

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-DETERMINE_LEVEL_INTE")
    def test_determine_level_intermediate_boundary(self) -> None:
        """エピソード数20件は中級者レベルの境界値"""
        # Given
        completed_episodes = 20
        average_quality_score = 70.0

        # When
        level = WriterLevelService.determine_level(completed_episodes, average_quality_score)

        # Then
        assert level == WriterLevel.INTERMEDIATE

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-DETERMINE_LEVEL_ADVA")
    def test_determine_level_advanced_with_many_episodes(self) -> None:
        """エピソード数が21-50の場合は上級者レベルと判定される"""
        # Given
        completed_episodes = 35
        average_quality_score = 70.0

        # When
        level = WriterLevelService.determine_level(completed_episodes, average_quality_score)

        # Then
        assert level == WriterLevel.ADVANCED

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-DETERMINE_LEVEL_ADVA")
    def test_determine_level_advanced_boundary(self) -> None:
        """エピソード数50件は上級者レベルの境界値"""
        # Given
        completed_episodes = 50
        average_quality_score = 70.0

        # When
        level = WriterLevelService.determine_level(completed_episodes, average_quality_score)

        # Then
        assert level == WriterLevel.ADVANCED

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-DETERMINE_LEVEL_EXPE")
    def test_determine_level_expert_with_many_episodes(self) -> None:
        """エピソード数が51以上の場合はエキスパートレベルと判定される"""
        # Given
        completed_episodes = 100
        average_quality_score = 70.0

        # When
        level = WriterLevelService.determine_level(completed_episodes, average_quality_score)

        # Then
        assert level == WriterLevel.EXPERT

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-DETERMINE_LEVEL_UPGR")
    def test_determine_level_upgrade_with_high_score(self) -> None:
        """平均スコアが85以上の場合は1段階レベルアップ"""
        # Given: 本来は初心者レベルのエピソード数
        completed_episodes = 3
        average_quality_score = 90.0

        # When
        level = WriterLevelService.determine_level(completed_episodes, average_quality_score)

        # Then: 中級者にレベルアップ
        assert level == WriterLevel.INTERMEDIATE

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-DETERMINE_LEVEL_DOWN")
    def test_determine_level_downgrade_with_low_score(self) -> None:
        """平均スコアが60未満の場合は1段階レベルダウン"""
        # Given: 本来は中級者レベルのエピソード数
        completed_episodes = 10
        average_quality_score = 50.0

        # When
        level = WriterLevelService.determine_level(completed_episodes, average_quality_score)

        # Then: 初心者にレベルダウン
        assert level == WriterLevel.BEGINNER

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-DETERMINE_LEVEL_NO_U")
    def test_determine_level_no_upgrade_for_expert(self) -> None:
        """エキスパートレベルは高スコアでもそれ以上にならない"""
        # Given
        completed_episodes = 100
        average_quality_score = 95.0

        # When
        level = WriterLevelService.determine_level(completed_episodes, average_quality_score)

        # Then
        assert level == WriterLevel.EXPERT

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-DETERMINE_LEVEL_NO_D")
    def test_determine_level_no_downgrade_for_beginner(self) -> None:
        """初心者レベルは低スコアでもそれ以下にならない"""
        # Given
        completed_episodes = 3
        average_quality_score = 30.0

        # When
        level = WriterLevelService.determine_level(completed_episodes, average_quality_score)

        # Then
        assert level == WriterLevel.BEGINNER

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-GET_LEVEL_DESCRIPTIO")
    def test_get_level_description_beginner(self) -> None:
        """初心者レベルの説明文を取得"""
        # When
        description = WriterLevelService.get_level_description(WriterLevel.BEGINNER)

        # Then
        assert description == "初心者 - 基礎を学びながら執筆を楽しみましょう"

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-GET_LEVEL_DESCRIPTIO")
    def test_get_level_description_intermediate(self) -> None:
        """中級者レベルの説明文を取得"""
        # When
        description = WriterLevelService.get_level_description(WriterLevel.INTERMEDIATE)

        # Then
        assert description == "中級者 - 安定した品質で執筆できています"

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-GET_LEVEL_DESCRIPTIO")
    def test_get_level_description_advanced(self) -> None:
        """上級者レベルの説明文を取得"""
        # When
        description = WriterLevelService.get_level_description(WriterLevel.ADVANCED)

        # Then
        assert description == "上級者 - 高い品質を維持しています"

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-GET_LEVEL_DESCRIPTIO")
    def test_get_level_description_expert(self) -> None:
        """エキスパートレベルの説明文を取得"""
        # When
        description = WriterLevelService.get_level_description(WriterLevel.EXPERT)

        # Then
        assert description == "エキスパート - プロ級の執筆技術です"

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-GET_ENCOURAGEMENT_ME")
    def test_get_encouragement_message_beginner_high_score(self) -> None:
        """初心者で高スコアの場合の励ましメッセージ"""
        # When
        message = WriterLevelService.get_encouragement_message(WriterLevel.BEGINNER, 65.0)

        # Then
        assert message == "素晴らしい進歩です!この調子で続けましょう。"

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-GET_ENCOURAGEMENT_ME")
    def test_get_encouragement_message_beginner_low_score(self) -> None:
        """初心者で低スコアの場合の励ましメッセージ"""
        # When
        message = WriterLevelService.get_encouragement_message(WriterLevel.BEGINNER, 55.0)

        # Then
        assert message == "少しずつ改善していきましょう。基礎をしっかり身につけることが大切です。"

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-GET_ENCOURAGEMENT_ME")
    def test_get_encouragement_message_intermediate_high_score(self) -> None:
        """中級者で高スコアの場合の励ましメッセージ"""
        # When
        message = WriterLevelService.get_encouragement_message(WriterLevel.INTERMEDIATE, 80.0)

        # Then
        assert message == "安定した品質です。より高みを目指しましょう!"

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-GET_ENCOURAGEMENT_ME")
    def test_get_encouragement_message_intermediate_low_score(self) -> None:
        """中級者で低スコアの場合の励ましメッセージ"""
        # When
        message = WriterLevelService.get_encouragement_message(WriterLevel.INTERMEDIATE, 70.0)

        # Then
        assert message == "もう少しで次のレベルです。細部にこだわってみましょう。"

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-GET_ENCOURAGEMENT_ME")
    def test_get_encouragement_message_advanced_high_score(self) -> None:
        """上級者で高スコアの場合の励ましメッセージ"""
        # When
        message = WriterLevelService.get_encouragement_message(WriterLevel.ADVANCED, 90.0)

        # Then
        assert message == "プロ級の品質に近づいています!"

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-GET_ENCOURAGEMENT_ME")
    def test_get_encouragement_message_advanced_low_score(self) -> None:
        """上級者で低スコアの場合の励ましメッセージ"""
        # When
        message = WriterLevelService.get_encouragement_message(WriterLevel.ADVANCED, 80.0)

        # Then
        assert message == "高い水準を維持しています。さらなる洗練を目指しましょう。"

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-GET_ENCOURAGEMENT_ME")
    def test_get_encouragement_message_expert(self) -> None:
        """エキスパートレベルの励ましメッセージ"""
        # When
        message = WriterLevelService.get_encouragement_message(WriterLevel.EXPERT, 95.0)

        # Then
        assert message == "最高水準の執筆です。あなたの作品は多くの読者を魅了するでしょう。"

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-DETERMINE_LEVEL_EDGE")
    def test_determine_level_edge_cases(self) -> None:
        """エッジケースのテスト"""
        # エピソード数0でも正しく動作
        level = WriterLevelService.determine_level(0, 70.0)
        assert level == WriterLevel.BEGINNER

        # スコア0でも正しく動作
        level = WriterLevelService.determine_level(10, 0.0)
        assert level == WriterLevel.BEGINNER

        # スコア100でも正しく動作
        level = WriterLevelService.determine_level(10, 100.0)
        assert level == WriterLevel.ADVANCED

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-LEVEL_ADJUSTMENT_WIT")
    def test_level_adjustment_with_boundary_scores(self) -> None:
        """境界値スコアでのレベル調整テスト"""
        # スコア60ちょうどは調整なし
        level = WriterLevelService.determine_level(10, 60.0)
        assert level == WriterLevel.INTERMEDIATE

        # スコア59.9は1段階ダウン
        level = WriterLevelService.determine_level(10, 59.9)
        assert level == WriterLevel.BEGINNER

        # スコア85ちょうどは1段階アップ
        level = WriterLevelService.determine_level(10, 85.0)
        assert level == WriterLevel.ADVANCED

        # スコア84.9は調整なし
        level = WriterLevelService.determine_level(10, 84.9)
        assert level == WriterLevel.INTERMEDIATE

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-CALCULATE_PROGRESS_T")
    def test_calculate_progress_to_next_level_beginner(self) -> None:
        """初心者レベルの進捗計算"""
        # Given
        completed_episodes = 3
        current_level = WriterLevel.BEGINNER

        # When
        progress = WriterLevelService.calculate_progress_to_next_level(completed_episodes, current_level)

        # Then
        assert progress["current_episodes"] == 3
        assert progress["next_level_threshold"] == 6
        assert progress["progress_percentage"] == 50.0  # 3/6 = 50%

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-CALCULATE_PROGRESS_T")
    def test_calculate_progress_to_next_level_intermediate(self) -> None:
        """中級者レベルの進捗計算"""
        # Given
        completed_episodes = 15
        current_level = WriterLevel.INTERMEDIATE

        # When
        progress = WriterLevelService.calculate_progress_to_next_level(completed_episodes, current_level)

        # Then
        assert progress["current_episodes"] == 15
        assert progress["next_level_threshold"] == 21
        assert progress["progress_percentage"] == 60.0  # (15-6)/(21-6) = 9/15 = 60%

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-CALCULATE_PROGRESS_T")
    def test_calculate_progress_to_next_level_advanced(self) -> None:
        """上級者レベルの進捗計算"""
        # Given
        completed_episodes = 35
        current_level = WriterLevel.ADVANCED

        # When
        progress = WriterLevelService.calculate_progress_to_next_level(completed_episodes, current_level)

        # Then
        assert progress["current_episodes"] == 35
        assert progress["next_level_threshold"] == 51
        assert round(progress["progress_percentage"], 2) == 46.67  # (35-21)/(51-21) ≈ 46.67%

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-CALCULATE_PROGRESS_T")
    def test_calculate_progress_to_next_level_expert(self) -> None:
        """エキスパートレベルの進捗計算(最高レベル)"""
        # Given
        completed_episodes = 100
        current_level = WriterLevel.EXPERT

        # When
        progress = WriterLevelService.calculate_progress_to_next_level(completed_episodes, current_level)

        # Then
        assert progress["current_episodes"] == 100
        assert progress["next_level_threshold"] is None
        assert progress["progress_percentage"] == 100.0

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-CALCULATE_PROGRESS_A")
    def test_calculate_progress_at_level_start(self) -> None:
        """レベル開始時点での進捗計算"""
        # 中級者レベルの開始時点
        progress = WriterLevelService.calculate_progress_to_next_level(6, WriterLevel.INTERMEDIATE)
        assert progress["progress_percentage"] == 0.0

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-CALCULATE_PROGRESS_A")
    def test_calculate_progress_at_level_end(self) -> None:
        """レベル終了時点での進捗計算"""
        # 中級者レベルの終了時点
        progress = WriterLevelService.calculate_progress_to_next_level(20, WriterLevel.INTERMEDIATE)
        assert round(progress["progress_percentage"], 2) == 93.33  # (20-6)/(21-6) ≈ 93.33%

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-CALCULATE_PROGRESS_O")
    def test_calculate_progress_over_threshold(self) -> None:
        """しきい値を超えた場合の進捗計算"""
        # 本来は次のレベルになるべきエピソード数
        progress = WriterLevelService.calculate_progress_to_next_level(25, WriterLevel.INTERMEDIATE)
        assert progress["progress_percentage"] == 100.0  # 最大100%

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-GENERATE_LEVEL_STATI")
    def test_generate_level_statistics_empty_data(self) -> None:
        """空データでの統計生成"""
        # When
        stats = WriterLevelService.generate_level_statistics([])

        # Then
        assert stats["total_episodes"] == 0
        assert stats["average_score"] == 0.0
        assert stats["current_level"] == WriterLevel.BEGINNER
        assert stats["score_trend"] == "no_data"

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-GENERATE_LEVEL_STATI")
    def test_generate_level_statistics_single_episode(self) -> None:
        """単一エピソードでの統計生成"""
        # Given
        episodes_data = [{"quality_score": 75.0}]

        # When
        stats = WriterLevelService.generate_level_statistics(episodes_data)

        # Then
        assert stats["total_episodes"] == 1
        assert stats["average_score"] == 75.0
        assert stats["current_level"] == WriterLevel.BEGINNER
        assert stats["score_trend"] == "insufficient_data"

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-GENERATE_LEVEL_STATI")
    def test_generate_level_statistics_improving_trend(self) -> None:
        """改善傾向の統計生成"""
        # Given
        episodes_data = [
            {"quality_score": 60.0},
            {"quality_score": 65.0},
            {"quality_score": 70.0},
            {"quality_score": 75.0},
            {"quality_score": 80.0},
        ]

        # When
        stats = WriterLevelService.generate_level_statistics(episodes_data)

        # Then
        assert stats["total_episodes"] == 5
        assert stats["average_score"] == 70.0
        assert stats["current_level"] == WriterLevel.BEGINNER
        assert stats["score_trend"] == "improving"

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-GENERATE_LEVEL_STATI")
    def test_generate_level_statistics_declining_trend(self) -> None:
        """低下傾向の統計生成"""
        # Given
        episodes_data = [
            {"quality_score": 80.0},
            {"quality_score": 75.0},
            {"quality_score": 70.0},
            {"quality_score": 65.0},
            {"quality_score": 60.0},
        ]

        # When
        stats = WriterLevelService.generate_level_statistics(episodes_data)

        # Then
        assert stats["score_trend"] == "declining"

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-GENERATE_LEVEL_STATI")
    def test_generate_level_statistics_stable_trend(self) -> None:
        """安定傾向の統計生成"""
        # Given
        episodes_data = [
            {"quality_score": 75.0},
            {"quality_score": 76.0},
            {"quality_score": 74.0},
            {"quality_score": 75.0},
            {"quality_score": 76.0},
        ]

        # When
        stats = WriterLevelService.generate_level_statistics(episodes_data)

        # Then
        assert stats["score_trend"] == "stable"

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-GENERATE_LEVEL_STATI")
    def test_generate_level_statistics_many_episodes(self) -> None:
        """多数エピソードでの統計生成"""
        # Given
        episodes_data = [{"quality_score": 85.0} for _ in range(25)]

        # When
        stats = WriterLevelService.generate_level_statistics(episodes_data)

        # Then
        assert stats["total_episodes"] == 25
        assert stats["average_score"] == 85.0
        assert stats["current_level"] == WriterLevel.EXPERT  # 25エピソード(ADVANCED) + 高スコア85でレベルアップ
        assert "エキスパート" in stats["level_description"]

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-GENERATE_LEVEL_STATI")
    def test_generate_level_statistics_missing_scores(self) -> None:
        """スコアが欠けているエピソードの処理"""
        # Given
        episodes_data = [
            {"quality_score": 80.0},
            {"other_data": "value"},  # quality_scoreなし
            {"quality_score": 70.0},
        ]

        # When
        stats = WriterLevelService.generate_level_statistics(episodes_data)

        # Then
        assert stats["total_episodes"] == 3
        assert stats["average_score"] == 75.0  # (80+70)/2

    @pytest.mark.spec("SPEC-WRITER_LEVEL_SERVICE-GENERATE_LEVEL_STATI")
    def test_generate_level_statistics_recent_episodes_only(self) -> None:
        """最近のエピソードのみでトレンド分析"""
        # Given: 10エピソード、最後の5つが改善傾向
        episodes_data = [{"quality_score": 60.0} for _ in range(5)]
        episodes_data.extend(
            [
                {"quality_score": 70.0},
                {"quality_score": 72.0},
                {"quality_score": 74.0},
                {"quality_score": 76.0},
                {"quality_score": 78.0},
            ]
        )

        # When
        stats = WriterLevelService.generate_level_statistics(episodes_data)

        # Then
        assert stats["score_trend"] == "improving"  # 最後の5エピソードで判定



#!/usr/bin/env python3
import importlib


def test_main_explicit_exports_callable():
    mod = importlib.import_module("src.mcp_servers.noveler.main")
    names = [
        "execute_enhanced_get_writing_tasks",
        "execute_enhanced_execute_writing_step",
        "execute_enhanced_resume_from_partial_failure",
        "execute_get_check_tasks",
        "execute_check_step_command",
        "execute_get_check_status",
        "execute_get_check_history",
        "execute_generate_episode_preview",
    ]
    for n in names:
        assert hasattr(mod, n), f"{n} not exported from main"
        assert callable(getattr(mod, n)), f"{n} is not callable"


def test_main___all_contains_exports():
    mod = importlib.import_module("src.mcp_servers.noveler.main")
    exported = set(getattr(mod, "__all__", []))
    required = {
        "execute_enhanced_get_writing_tasks",
        "execute_enhanced_execute_writing_step",
        "execute_enhanced_resume_from_partial_failure",
        "execute_get_check_tasks",
        "execute_check_step_command",
        "execute_get_check_status",
        "execute_get_check_history",
        "execute_generate_episode_preview",
    }
    assert required.issubset(exported)




"""Domain.value_objects.langsmith_artifacts
Where: Domain value object module defining typed data structures.
What: Declares dataclasses and helpers used to pass structured domain data.
Why: Keeps domain data definitions consistent across services and entities.
"""

from __future__ import annotations

from collections.abc import Sequence
from dataclasses import dataclass, field
from datetime import datetime, timezone
from pathlib import Path
from typing import Any


@dataclass(slots=True)
class LangSmithRun:
    """LangSmithのRun情報を表現する値オブジェクト"""

    run_id: str
    name: str
    status: str
    error: str | None = None
    trace_url: str | None = None
    inputs: dict[str, Any] = field(default_factory=dict)
    outputs: dict[str, Any] = field(default_factory=dict)
    metadata: dict[str, Any] = field(default_factory=dict)
    tags: list[str] = field(default_factory=list)
    captured_at: datetime = field(default_factory=lambda: datetime.now(timezone.utc))

    @property
    def is_failure(self) -> bool:
        """エラーかどうかを判定"""
        return self.status.lower() not in {"success", "completed"}

    def to_dataset_entry(self, expected_behavior: str | None = None) -> dict[str, Any]:
        """データセット行へ変換"""
        return {
            "run_id": self.run_id,
            "name": self.name,
            "status": self.status,
            "trace_url": self.trace_url,
            "inputs": self.inputs,
            "observed_output": self.outputs,
            "expected_behavior": expected_behavior,
            "error": self.error,
            "captured_at": self.captured_at.isoformat(),
            "tags": self.tags,
            "metadata": self.metadata,
        }

    def headline(self) -> str:
        """サマリの見出しテキストを取得"""
        status_icon = "❌" if self.is_failure else "✅"
        return f"{status_icon} LangSmith Run {self.run_id} ({self.name})"


@dataclass(slots=True)
class LangSmithBugfixArtifacts:
    """生成された成果物のメタ情報"""

    run: LangSmithRun
    summary_path: Path
    prompt_path: Path
    dataset_entry_path: Path | None = None


@dataclass(slots=True)
class PatchResult:
    """パッチ適用結果"""

    applied: bool
    stdout: str
    stderr: str
    command: Sequence[str]


@dataclass(slots=True)
class VerificationResult:
    """検証コマンドの実行結果"""

    returncode: int
    stdout: str
    stderr: str
    command: Sequence[str]

    @property
    def succeeded(self) -> bool:
        """成功判定"""
        return self.returncode == 0



"""Smart Auto-Enhancement Use Case

SPEC-SAE-003: Smart Auto-Enhancement ユースケース仕様
- novel check コマンドのSmart Auto-Enhancement機能を実現
- ドメインサービスとインフラストラクチャを協調させる
- Clean Architecture原則に基づく実装
"""

import os
import traceback
from dataclasses import dataclass
from pathlib import Path

from noveler.domain.entities.smart_auto_enhancement import (
    EnhancementMode,
    EnhancementRequest,
    SmartAutoEnhancement,
)
from noveler.domain.value_objects.episode_number import EpisodeNumber
from noveler.domain.value_objects.project_info import ProjectInfo


@dataclass(frozen=True)
class SmartAutoEnhancementUseCaseRequest:
    """Smart Auto-Enhancement ユースケースリクエスト"""

    project_name: str
    episode_number: int
    mode: str = "smart_auto"  # "standard", "smart_auto", "detailed", "enhanced"
    skip_basic: bool = False
    skip_a31: bool = False
    skip_claude: bool = False
    auto_fix: bool = False
    show_detailed_review: bool = False
    rhythm_analysis: bool = False


@dataclass(frozen=True)
class SmartAutoEnhancementUseCaseResponse:
    """Smart Auto-Enhancement ユースケースレスポンス"""

    success: bool
    enhancement: SmartAutoEnhancement | None
    error_message: str | None
    execution_time_ms: float
    final_score: float | None
    improvements_count: int

    @property
    def is_smart_auto_executed(self) -> bool:
        """Smart Auto-Enhancement が実行されたか"""
        return self.enhancement is not None and self.enhancement.is_smart_auto_mode and self.enhancement.is_success()


class SmartAutoEnhancementUseCase:
    """Smart Auto-Enhancement ユースケース

    novel check コマンドのSmart Auto-Enhancement機能を実現する。
    ドメインサービスを活用し、統合的な品質チェックを提供する。
    """

    def __init__(
        self,
        logger_service: object,
        unit_of_work: object,
        enhancement_service: object | None = None,
        **kwargs: object,
    ) -> None:
        """初期化 - B20準拠

        Args:
            logger_service: ロガーサービス
            unit_of_work: Unit of Work
            enhancement_service: エンハンスメントサービス（オプション）
            **kwargs: AbstractUseCaseの引数
        """
        super().__init__(**kwargs)
        self._logger_service = logger_service
        self._unit_of_work = unit_of_work
        # Unit of Workからエピソードリポジトリを取得
        self._episode_repository = unit_of_work.episode_repository
        self._enhancement_service = enhancement_service

    async def execute(self, request: SmartAutoEnhancementUseCaseRequest) -> SmartAutoEnhancementUseCaseResponse:
        """Smart Auto-Enhancement を実行

        Args:
            request: ユースケースリクエスト

        Returns:
            実行結果レスポンス
        """
        try:
            # エピソード内容を取得
            episode_content = await self._get_episode_content(request.project_name, request.episode_number)

            # ドメインリクエストを作成
            enhancement_request = self._create_enhancement_request(request)

            # Smart Auto-Enhancement エンティティを作成
            enhancement = SmartAutoEnhancement(enhancement_request)

            # ドメインサービスで拡張チェックを実行
            completed_enhancement = await self._enhancement_service.execute_enhancement(enhancement, episode_content)

            # レスポンスを作成
            return self._create_response(completed_enhancement)

        except Exception as e:
            error_msg = f"{type(e).__name__}: {str(e) or '不明なエラー'}"
            traceback_msg = traceback.format_exc()
            full_error_msg = f"{error_msg}\n\nTraceback:\n{traceback_msg}"

            return SmartAutoEnhancementUseCaseResponse(
                success=False,
                enhancement=None,
                error_message=full_error_msg,
                execution_time_ms=0.0,
                final_score=None,
                improvements_count=0,
            )

    async def _get_episode_content(self, project_name: str, episode_number: int) -> str:
        """エピソード内容を取得"""
        try:
            return self._episode_repository.get_episode_content(project_name, episode_number)
        except Exception as e:
            msg = f"エピソード取得エラー: {e!s}"
            raise ValueError(msg) from e

    def _create_enhancement_request(self, request: SmartAutoEnhancementUseCaseRequest) -> EnhancementRequest:
        """ドメインリクエストを作成"""
        # モード変換
        mode_mapping = {
            "standard": EnhancementMode.STANDARD,
            "smart_auto": EnhancementMode.SMART_AUTO,
            "detailed": EnhancementMode.DETAILED,
            "enhanced": EnhancementMode.ENHANCED,
        }

        mode = mode_mapping.get(request.mode, EnhancementMode.SMART_AUTO)

        # Smart Auto-Enhancement のビジネスルール適用
        skip_basic = request.skip_basic
        skip_a31 = request.skip_a31
        skip_claude = request.skip_claude
        show_detailed_review = request.show_detailed_review

        if mode in [EnhancementMode.SMART_AUTO, EnhancementMode.ENHANCED]:
            # Smart Auto-Enhancement では全段階実行＋詳細表示
            skip_basic = False
            skip_a31 = False
            skip_claude = False
            show_detailed_review = True

        # PROJECT_ROOTから直接ProjectInfoを作成
        try:
            project_root_env = os.getenv("PROJECT_ROOT")
            if not project_root_env:
                msg = "PROJECT_ROOT環境変数が設定されていません"
                raise ValueError(msg)

            project_root = Path(project_root_env)
            config_path = project_root / "プロジェクト設定.yaml"

            if not config_path.exists():
                msg = f"プロジェクト設定ファイルが見つかりません: {config_path}"
                raise ValueError(msg)

            project_info = ProjectInfo(name=request.project_name, root_path=project_root, config_path=config_path)

        except Exception as e:
            msg = f"プロジェクト情報作成エラー: {e}"
            raise ValueError(msg) from e

        return EnhancementRequest(
            episode_number=EpisodeNumber(request.episode_number),
            project_info=project_info,
            mode=mode,
            skip_basic=skip_basic,
            skip_a31=skip_a31,
            skip_claude=skip_claude,
            auto_fix=request.auto_fix,
            show_detailed_review=show_detailed_review,
            rhythm_analysis=request.rhythm_analysis,
        )

    def _create_response(self, enhancement: SmartAutoEnhancement) -> SmartAutoEnhancementUseCaseResponse:
        """レスポンスを作成"""
        try:
            final_result = enhancement.get_final_result()
            final_score = None
            error_message = None

            if final_result:
                if final_result.get_final_score():
                    final_score = final_result.get_final_score().value
                error_message = final_result.error_message

            # final_resultがNoneでも、enhancementが失敗状態の場合はエラー情報を取得
            if not enhancement.is_success() and error_message is None:
                # 失敗した段階からエラーメッセージを取得
                from noveler.domain.entities.smart_auto_enhancement import EnhancementStage

                for stage in [
                    EnhancementStage.BASIC_CHECK,
                    EnhancementStage.A31_EVALUATION,
                    EnhancementStage.CLAUDE_ANALYSIS,
                    EnhancementStage.FAILED,
                ]:
                    stage_result = enhancement.get_stage_result(stage)
                    if stage_result and stage_result.error_message:
                        error_message = stage_result.error_message
                        break

                # それでもエラーメッセージがない場合のフォールバック
                if error_message is None:
                    error_message = "Smart Auto-Enhancement実行中に不明なエラーが発生しました"

            return SmartAutoEnhancementUseCaseResponse(
                success=enhancement.is_success(),
                enhancement=enhancement,
                error_message=error_message,
                execution_time_ms=enhancement.get_execution_duration_ms(),
                final_score=final_score,
                improvements_count=enhancement.get_total_improvements_count(),
            )
        except Exception as e:
            # _create_response自体でエラーが発生した場合のフォールバック
            return SmartAutoEnhancementUseCaseResponse(
                success=False,
                enhancement=None,
                error_message=f"レスポンス作成エラー: {e}",
                execution_time_ms=0.0,
                final_score=None,
                improvements_count=0,
            )

    def should_use_smart_auto_enhancement(self, request: SmartAutoEnhancementUseCaseRequest) -> bool:
        """Smart Auto-Enhancement の使用判定

        ビジネスルール:
        - mode が "smart_auto" または "enhanced"
        - または従来の "standard" でも全段階有効な場合は自動適用
        """
        if request.mode in ["smart_auto", "enhanced"]:
            return True

        # 従来モードでも全段階が有効な場合は Smart Auto-Enhancement を適用
        return bool(not (request.skip_basic and request.skip_a31 and request.skip_claude))



#!/usr/bin/env python3
"""Top-level pytest configuration."""

from __future__ import annotations

import os

import pytest

from noveler.infrastructure.logging.unified_logger import get_logger as _get_unified_logger


def pytest_ignore_collect(collection_path=None, path=None, config=None) -> bool:
    """Tell pytest to ignore certain directories during collection."""
    p_obj = collection_path or path
    if p_obj is None:
        return False
    p = str(p_obj)
    blocked = (
        "/archive/" in p or "\\archive\\" in p or
        "/backups/" in p or "\\backups\\" in p
    )
    return bool(blocked)

def pytest_runtest_setup(item: pytest.Item) -> None:
    """Conditionally skip SPEC-gated tests."""
    try:
        if (
            "test_spec_901_async_mcp_integration.py" in str(item.fspath)
            and os.environ.get("ENABLE_SPEC_901_TESTS", "0") not in {"1", "true", "TRUE"}
        ):
            pytest.skip("SPEC-901 tests are gated by ENABLE_SPEC_901_TESTS=1")

        for mark in item.iter_markers(name="spec"):
            args = getattr(mark, "args", ())
            if not args:
                continue
            spec_name = args[0]
            if (
                isinstance(spec_name, str)
                and spec_name.strip().upper() == "SPEC-901-DDD-REFACTORING"
                and os.environ.get("ENABLE_SPEC_901_TESTS", "0") not in {"1", "true", "TRUE"}
            ):
                pytest.skip("SPEC-901 tests are gated by ENABLE_SPEC_901_TESTS=1")
    except Exception:
        return


def pytest_sessionstart(session: pytest.Session) -> None:
    """Initialise the unified logger early so pytest env overrides apply."""
    try:
        _get_unified_logger(__name__)
    except Exception:
        return


def pytest_addoption(parser: pytest.Parser) -> None:
    """Register project-level CLI flags."""
    parser.addoption(
        "--llm-report",
        action="store_true",
        default=False,
        help="Enable LLM-oriented test reporting output.",
    )



"""
フィードバックシステム

執筆プロセスのユーザーフィードバック収集・表示機能。
品質評価、改善提案、インタラクティブな確認機能を提供。
"""

from collections.abc import Callable
from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import Any

from noveler.infrastructure.performance.comprehensive_performance_optimizer import performance_monitor
from noveler.presentation.shared.shared_utilities import _get_console


def get_console():
    return _get_console()


class FeedbackType(Enum):
    """フィードバックの種類"""
    CONFIRMATION = "confirmation"
    QUALITY_CHECK = "quality_check"
    IMPROVEMENT_SUGGESTION = "improvement_suggestion"
    USER_INPUT = "user_input"
    WARNING = "warning"
    ERROR = "error"
    SUCCESS = "success"


@dataclass
class FeedbackMessage:
    """フィードバックメッセージ"""
    feedback_type: FeedbackType
    title: str
    message: str
    step_id: int | None = None
    options: list[str] | None = None
    default_option: str | None = None
    timestamp: datetime | None = None
    requires_response: bool = False
    severity: str = "info"  # info, warning, error, success

    def __post_init__(self):
        if self.timestamp is None:
            self.timestamp = datetime.now()


class InteractiveFeedbackSystem:
    """インタラクティブフィードバックシステム

    執筆プロセス中のユーザーとのインタラクションを管理し、
    品質チェック、確認、改善提案などのフィードバックを提供する。
    """

    def __init__(self, episode_number: int) -> None:
        self.episode_number = episode_number
        self.console = get_console()
        self.feedback_history: list[FeedbackMessage] = []
        self.user_preferences = {
            "auto_confirm_low_risk": False,
            "detailed_explanations": True,
            "show_improvement_tips": True,
            "interactive_mode": True
        }

    @performance_monitor
    def request_confirmation(
        self,
        title: str,
        message: str,
        step_id: int | None = None,
        default: bool = True
    ) -> bool:
        """確認を要求"""
        feedback = FeedbackMessage(
            feedback_type=FeedbackType.CONFIRMATION,
            title=title,
            message=message,
            step_id=step_id,
            options=["はい", "いいえ"],
            default_option="はい" if default else "いいえ",
            requires_response=True
        )

        self._log_feedback(feedback)

        if not self.user_preferences["interactive_mode"]:
            return default

        self._display_feedback(feedback)

        # ユーザー入力の待機（実際のCLI環境では input() を使用）
        response = self._get_user_response(feedback.options, feedback.default_option)

        return response.lower() in ["はい", "yes", "y", "1"]

    @performance_monitor
    def show_quality_check_result(
        self,
        step_id: int,
        quality_score: float,
        issues: list[str],
        suggestions: list[str]
    ) -> dict[str, Any]:
        """品質チェック結果の表示"""

        # 品質レベルの判定
        quality_level = self._determine_quality_level(quality_score)

        feedback = FeedbackMessage(
            feedback_type=FeedbackType.QUALITY_CHECK,
            title=f"ステップ{step_id} 品質チェック結果",
            message=f"品質スコア: {quality_score:.1f}/100 ({quality_level})",
            step_id=step_id,
            severity=self._get_severity_from_score(quality_score)
        )

        self._log_feedback(feedback)
        self._display_quality_result(feedback, quality_score, issues, suggestions)

        # 改善が必要な場合のインタラクション
        requires_improvement = quality_score < 70
        user_choice = None

        if requires_improvement:
            user_choice = self._handle_quality_improvement_dialog(step_id, issues, suggestions)

        return {
            "quality_score": quality_score,
            "quality_level": quality_level,
            "issues": issues,
            "suggestions": suggestions,
            "requires_improvement": requires_improvement,
            "user_choice": user_choice
        }

    @performance_monitor
    def show_improvement_suggestions(
        self,
        step_id: int,
        suggestions: list[dict[str, str]]
    ) -> str | None:
        """改善提案の表示"""
        if not self.user_preferences["show_improvement_tips"] or not suggestions:
            return None

        feedback = FeedbackMessage(
            feedback_type=FeedbackType.IMPROVEMENT_SUGGESTION,
            title=f"ステップ{step_id} 改善提案",
            message=f"{len(suggestions)}個の改善提案があります",
            step_id=step_id
        )

        self._log_feedback(feedback)
        self._display_improvement_suggestions(feedback, suggestions)

        # ユーザーが改善提案を選択
        if self.user_preferences["interactive_mode"]:
            return self._select_improvement_suggestion(suggestions)

        return None

    @performance_monitor
    def show_warning(
        self,
        title: str,
        message: str,
        step_id: int | None = None,
        require_acknowledgment: bool = False
    ) -> bool:
        """警告の表示"""
        feedback = FeedbackMessage(
            feedback_type=FeedbackType.WARNING,
            title=title,
            message=message,
            step_id=step_id,
            severity="warning",
            requires_response=require_acknowledgment
        )

        self._log_feedback(feedback)
        self._display_feedback(feedback)

        if require_acknowledgment:
            return self.request_confirmation("警告を確認しましたか？", "続行しますか？")

        return True

    @performance_monitor
    def show_error(
        self,
        title: str,
        message: str,
        step_id: int | None = None,
        recovery_options: list[str] | None = None
    ) -> str | None:
        """エラーの表示と復旧オプション"""
        feedback = FeedbackMessage(
            feedback_type=FeedbackType.ERROR,
            title=title,
            message=message,
            step_id=step_id,
            options=recovery_options,
            severity="error",
            requires_response=bool(recovery_options)
        )

        self._log_feedback(feedback)
        self._display_feedback(feedback)

        if recovery_options:
            return self._get_user_response(recovery_options, recovery_options[0])

        return None

    @performance_monitor
    def show_success(
        self,
        title: str,
        message: str,
        step_id: int | None = None,
        details: dict[str, Any] | None = None
    ) -> None:
        """成功メッセージの表示"""
        feedback = FeedbackMessage(
            feedback_type=FeedbackType.SUCCESS,
            title=title,
            message=message,
            step_id=step_id,
            severity="success"
        )

        self._log_feedback(feedback)
        self._display_feedback(feedback)

        if details and self.user_preferences["detailed_explanations"]:
            self._display_success_details(details)

    @performance_monitor
    def collect_user_input(
        self,
        prompt: str,
        input_type: str = "text",
        validation_func: Callable | None = None
    ) -> Any:
        """ユーザー入力の収集"""
        feedback = FeedbackMessage(
            feedback_type=FeedbackType.USER_INPUT,
            title="ユーザー入力要求",
            message=prompt,
            requires_response=True
        )

        self._log_feedback(feedback)
        self._display_feedback(feedback)

        # 実際の実装ではここでユーザー入力を受け取る
        # 今回はモックとして処理
        return self._mock_user_input(input_type)

    def _display_feedback(self, feedback: FeedbackMessage) -> None:
        """フィードバックの表示"""
        # 色とアイコンの設定
        color_map = {
            "info": "blue",
            "warning": "yellow",
            "error": "red",
            "success": "green"
        }

        icon_map = {
            FeedbackType.CONFIRMATION: "❓",
            FeedbackType.QUALITY_CHECK: "📊",
            FeedbackType.IMPROVEMENT_SUGGESTION: "💡",
            FeedbackType.USER_INPUT: "✏️ ",
            FeedbackType.WARNING: "⚠️ ",
            FeedbackType.ERROR: "❌",
            FeedbackType.SUCCESS: "✅"
        }

        color = color_map.get(feedback.severity, "white")
        icon = icon_map.get(feedback.feedback_type, "ℹ️ ")

        step_info = f" [ステップ{feedback.step_id}]" if feedback.step_id else ""

        self.console.print(f"\n[bold {color}]{icon} {feedback.title}{step_info}[/bold {color}]")
        self.console.print(f"[{color}]{feedback.message}[/{color}]")

        if feedback.options:
            self.console.print(f"[dim]選択肢: {', '.join(feedback.options)}[/dim]")

    def _display_quality_result(
        self,
        feedback: FeedbackMessage,
        quality_score: float,
        issues: list[str],
        suggestions: list[str]
    ) -> None:
        """品質チェック結果の詳細表示"""
        self._display_feedback(feedback)

        # 品質バー
        quality_bar = self._create_quality_bar(quality_score)
        self.console.print(f"品質: {quality_bar} {quality_score:.1f}/100")

        # 問題点
        if issues:
            self.console.print(f"\n[red]検出された問題 ({len(issues)}件):[/red]")
            for i, issue in enumerate(issues[:5], 1):  # 最大5件表示
                self.console.print(f"  {i}. {issue}")
            if len(issues) > 5:
                self.console.print(f"  ... 他 {len(issues)-5} 件")

        # 改善提案
        if suggestions:
            self.console.print(f"\n[blue]改善提案 ({len(suggestions)}件):[/blue]")
            for i, suggestion in enumerate(suggestions[:3], 1):  # 最大3件表示
                self.console.print(f"  💡 {suggestion}")
            if len(suggestions) > 3:
                self.console.print(f"  ... 他 {len(suggestions)-3} 件")

    def _display_improvement_suggestions(
        self,
        feedback: FeedbackMessage,
        suggestions: list[dict[str, str]]
    ) -> None:
        """改善提案の表示"""
        self._display_feedback(feedback)

        for i, suggestion in enumerate(suggestions, 1):
            title = suggestion.get("title", f"提案 {i}")
            description = suggestion.get("description", "")
            impact = suggestion.get("impact", "medium")

            impact_icon = {"high": "🔥", "medium": "⭐", "low": "💫"}.get(impact, "⭐")

            self.console.print(f"\n{impact_icon} [bold]{title}[/bold]")
            self.console.print(f"   {description}")

    def _display_success_details(self, details: dict[str, Any]) -> None:
        """成功の詳細情報表示"""
        self.console.print("\n[dim]詳細情報:[/dim]")
        for key, value in details.items():
            self.console.print(f"  • {key}: {value}")

    def _handle_quality_improvement_dialog(
        self,
        step_id: int,
        issues: list[str],
        suggestions: list[str]
    ) -> str:
        """品質改善ダイアログの処理"""
        options = [
            "自動修正を適用",
            "手動で修正",
            "このまま続行",
            "ステップをスキップ"
        ]

        self.console.print(f"\n[yellow]ステップ{step_id}の品質に問題があります。どうしますか？[/yellow]")

        return self._get_user_response(options, options[0])

    def _select_improvement_suggestion(self, suggestions: list[dict[str, str]]) -> str:
        """改善提案の選択"""
        if len(suggestions) == 1:
            return suggestions[0].get("title", "")

        options = [s.get("title", f"提案{i+1}") for i, s in enumerate(suggestions)]
        options.append("提案を適用しない")

        self.console.print("\n[blue]適用する改善提案を選択してください:[/blue]")

        return self._get_user_response(options, options[0])

    def _get_user_response(self, options: list[str], default: str) -> str:
        """ユーザーレスポンスの取得（モック実装）"""
        # 実際の実装では input() を使用してユーザー入力を受け取る
        # 今回は自動的にデフォルト選択を返す
        return default

    def _mock_user_input(self, input_type: str) -> Any:
        """ユーザー入力のモック"""
        mock_inputs = {
            "text": "ユーザー入力テキスト",
            "number": 42,
            "boolean": True
        }
        return mock_inputs.get(input_type, "デフォルト値")

    def _create_quality_bar(self, score: float, width: int = 30) -> str:
        """品質バーの作成"""
        filled = int((score / 100) * width)

        # 色分け
        if score >= 80:
            bar_char = "█"
            color = "green"
        elif score >= 60:
            bar_char = "█"
            color = "yellow"
        else:
            bar_char = "█"
            color = "red"

        bar = bar_char * filled + "░" * (width - filled)
        return f"[{color}][{bar}][/{color}]"

    def _determine_quality_level(self, score: float) -> str:
        """品質レベルの判定"""
        if score >= 90:
            return "優秀"
        if score >= 80:
            return "良好"
        if score >= 70:
            return "合格"
        if score >= 60:
            return "要改善"
        return "不合格"

    def _get_severity_from_score(self, score: float) -> str:
        """スコアから深刻度を取得"""
        if score >= 80:
            return "success"
        if score >= 60:
            return "warning"
        return "error"

    def _log_feedback(self, feedback: FeedbackMessage) -> None:
        """フィードバックのログ記録"""
        self.feedback_history.append(feedback)

    @performance_monitor
    def export_feedback_report(self) -> dict[str, Any]:
        """フィードバックレポートのエクスポート"""
        return {
            "episode_number": self.episode_number,
            "timestamp": datetime.now().isoformat(),
            "total_feedback_count": len(self.feedback_history),
            "feedback_by_type": {
                feedback_type.value: len([f for f in self.feedback_history if f.feedback_type == feedback_type])
                for feedback_type in FeedbackType
            },
            "feedback_history": [
                {
                    "type": f.feedback_type.value,
                    "title": f.title,
                    "message": f.message,
                    "step_id": f.step_id,
                    "timestamp": f.timestamp.isoformat() if f.timestamp else None,
                    "severity": f.severity
                }
                for f in self.feedback_history
            ],
            "user_preferences": self.user_preferences
        }

    def set_user_preferences(self, preferences: dict[str, Any]) -> None:
        """ユーザー設定の更新"""
        self.user_preferences.update(preferences)

    def get_feedback_summary(self) -> dict[str, Any]:
        """フィードバック サマリーの取得"""
        return {
            "total_feedback": len(self.feedback_history),
            "confirmations": len([f for f in self.feedback_history if f.feedback_type == FeedbackType.CONFIRMATION]),
            "quality_checks": len([f for f in self.feedback_history if f.feedback_type == FeedbackType.QUALITY_CHECK]),
            "warnings": len([f for f in self.feedback_history if f.feedback_type == FeedbackType.WARNING]),
            "errors": len([f for f in self.feedback_history if f.feedback_type == FeedbackType.ERROR]),
            "last_feedback": self.feedback_history[-1] if self.feedback_history else None
        }



#!/usr/bin/env python3

"""Domain.entities.quality_record
Where: Domain entity storing quality record aggregates.
What: Captures quality metrics, history, and annotations.
Why: Provides a source of truth for quality tracking.
"""

from __future__ import annotations

"""品質記録ドメインエンティティ
記録の永続化とビジネスルールを管理
"""


from dataclasses import dataclass
from datetime import datetime, timedelta
from typing import TYPE_CHECKING, Any
from uuid import uuid4

from noveler.domain.exceptions import QualityRecordError
from noveler.domain.value_objects.project_time import ProjectTimezone, project_now
from noveler.domain.value_objects.quality_check_result import QualityCheckResult, QualityScore

# JSTタイムゾーン
JST = ProjectTimezone.jst().timezone


@dataclass
class QualityRecordEntry:
    """品質記録エントリ(イミュータブル)"""

    id: str
    quality_result: QualityCheckResult
    created_at: datetime
    metadata: dict[str, Any] | None = None

    def __post_init__(self) -> None:
        if not self.id:
            object.__setattr__(self, "id", str(uuid4()))

    @classmethod
    def create_from_result(
        cls, result: QualityCheckResult, metadata: dict[str, Any] | None = None
    ) -> QualityRecordEntry:
        """品質チェック結果からエントリを作成"""
        return cls(id=str(uuid4()), quality_result=result, created_at=project_now().datetime, metadata=metadata or {})


class QualityRecord:
    """品質記録ドメインエンティティ(アグリゲートルート)"""

    def __init__(self, project_name: str, entries: list[QualityRecordEntry] | None = None) -> None:
        self._project_name = self._validate_project_name(project_name)
        self._entries: list[QualityRecordEntry] = list(entries) if entries else []
        self._last_updated = project_now().datetime

        # ドメインイベント記録
        self._domain_events: list[Any] = []

    @staticmethod
    def _validate_project_name(project_name: str) -> str:
        """プロジェクト名の妥当性チェック"""
        if not project_name or not project_name.strip():
            msg = "Project name cannot be empty"
            raise QualityRecordError(None, msg)
        return project_name.strip()

    @property
    def project_name(self) -> str:
        """プロジェクト名を取得する。

        Returns:
            str: プロジェクト名
        """
        return self._project_name

    @property
    def last_updated(self) -> datetime:
        """最終更新日時を取得する。

        Returns:
            datetime: 最終更新日時
        """
        return self._last_updated

    @property
    def entries(self) -> list[QualityRecordEntry]:
        """記録エントリのコピーを返す(不変性保証)"""
        return list(self._entries)

    @property
    def entry_count(self) -> int:
        """記録エントリ数"""
        return len(self._entries)

    def add_quality_check_result(self, result: QualityCheckResult, metadata: dict[str, Any] | None = None) -> str:
        """品質チェック結果を追加"""
        # ビジネスルール: 同じエピソード・同じ時刻の重複チェック防止
        if self._has_duplicate_entry(result):
            msg = f"Duplicate quality check for episode {result.episode_number} at {result.timestamp}"
            raise QualityRecordError(None, msg)

        entry = QualityRecordEntry.create_from_result(result, metadata)
        self._entries.append(entry)
        now = project_now().datetime
        self._last_updated = now

        # ドメインイベント発行
        overall_score = entry.quality_result.overall_score.to_float()
        self._domain_events.append(
            {
                "type": "QualityCheckAdded",
                "entry_id": entry.id,
                "episode_number": entry.quality_result.episode_number,
                "score": overall_score,
                "timestamp": now,
            }
        )

        return entry.id

    def _has_duplicate_entry(self, result: QualityCheckResult) -> bool:
        """重複エントリチェック"""
        for entry in self._entries:
            if (
                entry.quality_result.episode_number == result.episode_number
                and abs((entry.quality_result.timestamp - result.timestamp).total_seconds()) < 60
            ):
                return True
        return False

    def get_latest_for_episode(self, episode_number: int) -> QualityRecordEntry | None:
        """指定エピソードの最新記録を取得"""
        episode_entries = [e for e in self._entries if e.quality_result.episode_number == episode_number]
        if not episode_entries:
            return None

        return max(episode_entries, key=lambda e: e.created_at)

    def get_quality_trend(self, episode_number: int, limit: int | None = None) -> list[QualityScore]:
        """品質スコアのトレンドを取得"""
        episode_entries = [e for e in self._entries if e.quality_result.episode_number == episode_number]

        # 時系列順でソート
        sorted_entries = sorted(episode_entries, key=lambda e: e.created_at)

        # 指定件数に制限
        if limit is not None and limit > 0:
            recent_entries = sorted_entries[-limit:]
        else:
            recent_entries = sorted_entries

        return [entry.quality_result.overall_score for entry in recent_entries]

    def calculate_average_score(self) -> QualityScore | None:
        """全エピソードの平均品質スコア"""
        if not self._entries:
            return None

        total_score = sum(entry.quality_result.overall_score.value for entry in self._entries)
        average = total_score / len(self._entries)
        return QualityScore(average)

    def get_episodes_below_threshold(self, threshold: QualityScore | None = None) -> list[int]:
        """閾値以下の品質スコアのエピソード一覧"""
        effective_threshold = threshold or QualityScore.from_float(80.0)

        low_quality_episodes = set()
        for entry in self._entries:
            if entry.quality_result.overall_score.value < effective_threshold.value:
                low_quality_episodes.add(entry.quality_result.episode_number)

        return sorted(low_quality_episodes)

    def purge_old_entries(self, days_to_keep: int) -> int:
        """古い記録のパージ(保持期間制御)"""
        if days_to_keep <= 0:
            msg = "Days to keep must be positive"
            raise QualityRecordError(None, msg)

        now = datetime.now(JST)
        cutoff_date = now.replace(hour=0, minute=0, second=0, microsecond=0) - timedelta(days=days_to_keep)

        original_count = len(self._entries)
        self._entries = [e for e in self._entries if e.created_at >= cutoff_date]
        purged_count = original_count - len(self._entries)

        if purged_count > 0:
            self._last_updated = now
            self._domain_events.append(
                {"type": "OldEntriesPurged", "purged_count": purged_count, "timestamp": now}
            )

        return purged_count

    def get_domain_events(self) -> list[Any]:
        """ドメインイベント取得"""
        return list(self._domain_events)

    def clear_domain_events(self) -> None:
        """ドメインイベントクリア"""
        self._domain_events.clear()

    def to_persistence_dict(self) -> dict[str, Any]:
        """永続化用辞書変換"""
        return {
            "metadata": {
                "project_name": self._project_name,
                "last_updated": self._last_updated.isoformat(),
                "entry_count": len(self._entries),
            },
            "quality_checks": [
                {
                    "id": entry.id,
                    "episode_number": entry.quality_result.episode_number,
                    "timestamp": entry.quality_result.timestamp.isoformat(),
                    "checker_version": entry.quality_result.checker_version,
                    "results": entry.quality_result.to_summary_dict(),
                    "created_at": entry.created_at.isoformat(),
                    "metadata": entry.metadata,
                }
                for entry in self._entries
            ],
        }



#!/usr/bin/env python3

"""Domain.entities.learning_session
Where: Domain entity tracking learning sessions.
What: Records learning objectives, activities, and outcomes.
Why: Supports analytics and progress tracking for learning workflows.
"""

from __future__ import annotations

"""学習セッションエンティティ
品質記録活用システムのドメインモデル
"""


from dataclasses import dataclass
from typing import TYPE_CHECKING, Any

from noveler.domain.exceptions import BusinessRuleViolationError
from noveler.domain.value_objects.learning_metrics import LearningMetrics
from noveler.domain.value_objects.project_time import ProjectTimezone, project_now

if TYPE_CHECKING:
    from datetime import datetime

# JSTタイムゾーン
JST = ProjectTimezone.jst().timezone


@dataclass
class LearningSession:
    """学習セッションエンティティ

    個別の執筆セッションのコンテキストと学習データを管理
    """

    project_name: str
    episode_number: int
    start_time: datetime
    writing_environment: str | None = None
    target_audience: str | None = None
    writing_goal: str | None = None
    end_time: datetime | None = None
    total_writing_time: int = 0  # 分単位
    is_completed: bool = False

    def __post_init__(self) -> None:
        """エンティティの不変条件を検証"""
        self._validate_project_name()
        self._validate_episode_number()
        self._validate_start_time()

    def _validate_project_name(self) -> None:
        """プロジェクト名の妥当性検証"""
        if not self.project_name or len(self.project_name.strip()) == 0:
            msg = "プロジェクト名は必須です"
            error_code = "project_name_required"
            raise BusinessRuleViolationError(error_code, msg)

    def _validate_episode_number(self) -> None:
        """エピソード番号の妥当性検証"""
        if self.episode_number <= 0:
            msg = "エピソード番号は1以上の正の整数である必要があります"
            error_code = "episode_number_invalid"
            raise BusinessRuleViolationError(error_code, msg)

    def _validate_start_time(self) -> None:
        """開始時刻の妥当性検証"""
        if self.start_time is None:
            msg = "開始時刻は必須です"
            error_code = "start_time_required"
            raise BusinessRuleViolationError(error_code, msg)

    def complete(self, end_time: datetime | None = None) -> None:
        """学習セッションを完了"""
        if self.is_completed:
            msg = "既に完了したセッションです"
            error_code = "session_already_completed"
            raise BusinessRuleViolationError(error_code, msg)

        self.end_time = end_time or project_now().datetime

        # 開始時刻より前に終了することはできない
        if self.end_time < self.start_time:
            msg = "終了時刻は開始時刻より後である必要があります"
            error_code = "invalid_end_time"
            raise BusinessRuleViolationError(error_code, msg)

        # 執筆時間を計算(分単位)
        duration = self.end_time - self.start_time
        self.total_writing_time = int(duration.total_seconds() / 60)

        self.is_completed = True

    def get_session_duration(self) -> int:
        """セッションの継続時間を取得(分単位)"""
        if not self.is_completed:
            return 0
        return self.total_writing_time

    def get_session_context(self) -> dict[str, Any]:
        """セッションのコンテキストを取得"""
        return {
            "project_name": self.project_name,
            "episode_number": self.episode_number,
            "writing_environment": self.writing_environment,
            "target_audience": self.target_audience,
            "writing_goal": self.writing_goal,
            "start_time": self.start_time.isoformat(),
            "end_time": self.end_time.isoformat() if self.end_time else None,
            "total_writing_time": self.total_writing_time,
            "is_completed": self.is_completed,
        }

    def create_learning_metrics(
        self, improvement_from_previous: float, revision_count: int, user_feedback: str | None = None
    ) -> LearningMetrics:
        """学習メトリクスを作成"""
        if not self.is_completed:
            msg = "セッションが完了していません"
            error_code = "session_not_completed"
            raise BusinessRuleViolationError(error_code, msg)

        return LearningMetrics(
            improvement_from_previous=improvement_from_previous,
            time_spent_writing=self.total_writing_time,
            revision_count=revision_count,
            user_feedback=user_feedback,
            writing_context=self.writing_environment,
        )

    def is_long_session(self, threshold_minutes: int = 120) -> bool:
        """長時間セッションかどうかを判定"""
        return self.total_writing_time >= threshold_minutes

    def is_short_session(self, threshold_minutes: int = 30) -> bool:
        """短時間セッションかどうかを判定"""
        return self.total_writing_time <= threshold_minutes

    def get_productivity_level(self) -> str:
        """生産性レベルを取得"""
        # 標準的な閾値を使用(120分を高生産性、30分を低生産性とする)
        if self.is_long_session(120):
            return "高生産性"
        if self.is_short_session(30):
            return "低生産性"
        return "標準生産性"



"""Simplified deploy scripts use case for unit tests."""

from __future__ import annotations

from collections.abc import Iterable
from dataclasses import dataclass, field

from noveler.domain.deployment.entities import (
    Deployment,
    DeploymentMode,
    DeploymentStatus,
    DeploymentTarget,
)
from noveler.domain.deployment.value_objects import CommitHash
from noveler.domain.value_objects.project_time import project_now


@dataclass
class DeployScriptsRequest:
    """Parameters that control how deployment scripts are executed.

    Attributes:
        targets: Optional collection of deployment targets to process.
        mode: Deployment mode that should be applied to every deployment.
        force: When True, continue despite validation failures.
    """

    targets: Iterable[DeploymentTarget] | None = None
    mode: DeploymentMode = DeploymentMode.PRODUCTION
    force: bool = False


@dataclass
class DeployScriptsResult:
    """Result returned by the deployment use case.

    Attributes:
        success: Indicates whether all deployments finished successfully.
        deployments: Deployment records generated during execution.
        total_time: Total time spent executing the deployments in seconds.
        error_message: Optional message describing a failure.
    """

    success: bool
    deployments: list[Deployment] = field(default_factory=list)
    total_time: float = 0.0
    error_message: str | None = None


class DeployScriptsUseCase:
    """Coordinate deployment service and repositories to deploy scripts."""

    def __init__(self, deployment_repo, deployment_service, version_service) -> None:
        self._deployment_repo = deployment_repo
        self._deployment_service = deployment_service
        self._version_service = version_service

    # ------------------------------------------------------------------
    # Entry point
    # ------------------------------------------------------------------
    def execute(self, request: DeployScriptsRequest | None = None, **kwargs) -> DeployScriptsResult:
        """Execute deployments for the provided targets.

        Args:
            request: Deployment parameters; if omitted, kwargs are used.
            **kwargs: Fallback keyword arguments to construct a request.

        Returns:
            DeployScriptsResult: Aggregated deployment information.
        """
        if request is None:
            request = DeployScriptsRequest(**kwargs)

        start_time = project_now().datetime
        deployments: list[Deployment] = []

        try:
            targets = list(request.targets) if request.targets is not None else list(self._deployment_service.find_deployable_projects())
            if not targets:
                return DeployScriptsResult(success=False, error_message="No deployable projects found")

            for target in targets:
                deployment = self._deploy_to_target(target, request.mode, request.force)
                deployments.append(deployment)
                if deployment.status == DeploymentStatus.FAILED and not request.force:
                    self._rollback_deployments(deployments)
                    return DeployScriptsResult(
                        success=False,
                        deployments=deployments,
                        total_time=(project_now().datetime - start_time).total_seconds(),
                        error_message=deployment.error_message or "Deployment failed",
                    )

            total_time = (project_now().datetime - start_time).total_seconds()
            return DeployScriptsResult(success=True, deployments=deployments, total_time=total_time)
        except Exception as exc:  # pragma: no cover - defensive
            self._rollback_deployments(deployments)
            return DeployScriptsResult(
                success=False,
                deployments=deployments,
                total_time=(project_now().datetime - start_time).total_seconds(),
                error_message=str(exc),
            )

    # ------------------------------------------------------------------
    # Internal helpers
    # ------------------------------------------------------------------
    def _deploy_to_target(self, target: DeploymentTarget, mode: DeploymentMode, force: bool) -> Deployment:
        """Trigger a deployment workflow for a single target."""
        deployment = self._deployment_service.create_deployment(target, mode)

        is_ready, issues = self._deployment_service.validate_deployment_readiness(deployment)
        if not is_ready and not force:
            deployment.fail(f"Validation failed: {', '.join(issues)}")
            self._deployment_repo.save(deployment)
            return deployment

        try:
            deployment.start()
            self._deployment_repo.save(deployment)
            self._perform_deployment(deployment)
            deployment.complete()
        except Exception as exc:
            deployment.fail(str(exc))
        finally:
            self._deployment_repo.save(deployment)

        return deployment

    def _perform_deployment(self, deployment: Deployment) -> None:
        """Hook patched by tests; by default does nothing."""
        # Simulate minimal work by touching version info if available
        if hasattr(self._version_service, "get_current_commit"):
            commit = self._version_service.get_current_commit()
            if commit:
                deployment.source_commit = CommitHash(str(commit))

    def _rollback_deployments(self, deployments: Iterable[Deployment]) -> None:
        """Rollback deployments that finished before a failure."""
        for deployment in deployments:
            if deployment.status == DeploymentStatus.COMPLETED:
                deployment.status = DeploymentStatus.ROLLED_BACK
            elif deployment.status not in (DeploymentStatus.FAILED, DeploymentStatus.ROLLED_BACK):
                deployment.fail("Rolled back due to previous failure")
            self._deployment_repo.save(deployment)



#!/usr/bin/env python3
# File: tests/unit/domain/services/test_creative_intention_validator.py
# Purpose: Unit tests for CreativeIntentionValidator service
# Context: Validates 5-point creative intention validation logic

"""Tests for CreativeIntentionValidator service."""

import pytest

from noveler.domain.services.creative_intention_validator import (
    CreativeIntentionValidator,
    ValidationIssue,
    ValidationResult,
)
from noveler.domain.value_objects.creative_intention import (
    CharacterArc,
    CreativeIntention,
)


class TestValidationIssue:
    """Test ValidationIssue dataclass."""

    def test_validation_issue_creation(self):
        """Test creating a ValidationIssue."""
        issue = ValidationIssue(
            field_name="scene_goal",
            severity="error",
            message="Scene goal too short",
            suggestion="Add more detail"
        )

        assert issue.field_name == "scene_goal"
        assert issue.severity == "error"
        assert issue.message == "Scene goal too short"
        assert issue.suggestion == "Add more detail"


class TestValidationResult:
    """Test ValidationResult dataclass."""

    def test_has_errors_true(self):
        """Test has_errors returns True when errors exist."""
        result = ValidationResult(
            is_valid=False,
            issues=[
                ValidationIssue("field1", "error", "Error message"),
                ValidationIssue("field2", "warning", "Warning message")
            ]
        )

        assert result.has_errors() is True
        assert result.has_warnings() is True

    def test_has_errors_false(self):
        """Test has_errors returns False when only warnings exist."""
        result = ValidationResult(
            is_valid=True,
            issues=[
                ValidationIssue("field1", "warning", "Warning message")
            ]
        )

        assert result.has_errors() is False
        assert result.has_warnings() is True

    def test_get_error_count(self):
        """Test get_error_count returns correct count."""
        result = ValidationResult(
            is_valid=False,
            issues=[
                ValidationIssue("field1", "error", "Error 1"),
                ValidationIssue("field2", "error", "Error 2"),
                ValidationIssue("field3", "warning", "Warning")
            ]
        )

        assert result.get_error_count() == 2
        assert result.get_warning_count() == 1


class TestCreativeIntentionValidator:
    """Test CreativeIntentionValidator service."""

    @pytest.fixture
    def validator(self):
        """Return a CreativeIntentionValidator instance."""
        return CreativeIntentionValidator()

    @pytest.fixture
    def valid_arc(self):
        """Return a valid CharacterArc."""
        return CharacterArc(
            before_state="いじめられている被害者",
            transition="Brain Burstの授与",
            after_state="加速世界への転移"
        )

    @pytest.fixture
    def valid_intention(self, valid_arc):
        """Return a valid CreativeIntention."""
        return CreativeIntention(
            scene_goal="主人公の弱点を冒頭で明示し共感獲得",
            emotional_goal="絶望→驚き→期待",
            character_arc=valid_arc,
            world_via_action="時間停止を体験シーンで提示",
            voice_constraints="「AはBである」形式禁止"
        )

    def test_validate_valid_intention(self, validator, valid_intention):
        """Test validating a completely valid intention."""
        result = validator.validate(valid_intention)

        assert result.is_valid is True
        assert result.checked_intention == valid_intention
        # May have warnings but no errors
        assert result.get_error_count() == 0

    def test_validate_too_short_scene_goal(self, validator, valid_arc):
        """Test validation catches too-short scene_goal."""
        # This will raise ValueError in CreativeIntention.__post_init__
        with pytest.raises(ValueError, match="scene_goal must be at least"):
            intention = CreativeIntention(
                scene_goal="短すぎ",  # 3 chars - too short
                emotional_goal="絶望→驚き→期待",
                character_arc=valid_arc,
                world_via_action="時間停止を体験シーンで提示",
                voice_constraints="「AはBである」形式禁止"
            )

    def test_validate_brief_scene_goal_warning(self, validator, valid_arc):
        """Test validation warns for brief (but valid) scene_goal."""
        intention = CreativeIntention(
            scene_goal="最小限の10文字以上",  # 10 chars - valid but brief
            emotional_goal="絶望→驚き→期待",
            character_arc=valid_arc,
            world_via_action="時間停止を体験シーンで提示",
            voice_constraints="「AはBである」形式禁止"
        )

        result = validator.validate(intention)

        # Should pass but with warning
        assert result.is_valid is True
        assert result.get_warning_count() > 0

        # Check for scene_goal warning
        scene_goal_warnings = [
            issue for issue in result.issues
            if issue.field_name == "scene_goal" and issue.severity == "warning"
        ]
        assert len(scene_goal_warnings) > 0

    def test_validate_emotional_goal_without_arrow(self, validator, valid_arc):
        """Test validation warns when emotional_goal lacks progression arrow."""
        intention = CreativeIntention(
            scene_goal="主人公の弱点を冒頭で明示し共感獲得",
            emotional_goal="絶望と驚きと期待",  # No arrow - warning
            character_arc=valid_arc,
            world_via_action="時間停止を体験シーンで提示",
            voice_constraints="「AはBである」形式禁止"
        )

        result = validator.validate(intention)

        # Should pass but with warning about arrow
        assert result.is_valid is True
        emotional_warnings = [
            issue for issue in result.issues
            if issue.field_name == "emotional_goal" and "progression" in issue.message
        ]
        assert len(emotional_warnings) > 0

    def test_validate_world_via_action_with_exposition_keywords(self, validator, valid_arc):
        """Test validation warns for exposition keywords in world_via_action."""
        intention = CreativeIntention(
            scene_goal="主人公の弱点を冒頭で明示し共感獲得",
            emotional_goal="絶望→驚き→期待",
            character_arc=valid_arc,
            world_via_action="時間停止の仕組みを説明する",  # 説明 - warning
            voice_constraints="「AはBである」形式禁止"
        )

        result = validator.validate(intention)

        # Should pass but with warning
        assert result.is_valid is True
        world_warnings = [
            issue for issue in result.issues
            if issue.field_name == "world_via_action" and "exposition" in issue.message
        ]
        assert len(world_warnings) > 0

    def test_validate_voice_constraints_without_prohibition(self, validator, valid_arc):
        """Test validation warns when voice_constraints lack prohibition."""
        intention = CreativeIntention(
            scene_goal="主人公の弱点を冒頭で明示し共感獲得",
            emotional_goal="絶望→驚き→期待",
            character_arc=valid_arc,
            world_via_action="時間停止を体験シーンで提示",
            voice_constraints="三人称単一視点で書く"  # No prohibition - warning
        )

        result = validator.validate(intention)

        # Should pass but with warning
        assert result.is_valid is True
        voice_warnings = [
            issue for issue in result.issues
            if issue.field_name == "voice_constraints" and "prohibition" in issue.message
        ]
        assert len(voice_warnings) > 0

    def test_validate_for_step_11_with_episode_number(self, validator, valid_arc):
        """Test validate_for_step_11 with episode_number set."""
        intention = CreativeIntention(
            scene_goal="主人公の弱点を冒頭で明示し共感獲得",
            emotional_goal="絶望→驚き→期待",
            character_arc=valid_arc,
            world_via_action="時間停止を体験シーンで提示",
            voice_constraints="「AはBである」形式禁止",
            episode_number=1
        )

        result = validator.validate_for_step_11(intention)

        assert result.is_valid is True
        # Should not warn about episode_number
        episode_warnings = [
            issue for issue in result.issues
            if issue.field_name == "episode_number"
        ]
        assert len(episode_warnings) == 0

    def test_validate_for_step_11_without_episode_number(self, validator, valid_arc):
        """Test validate_for_step_11 warns without episode_number."""
        intention = CreativeIntention(
            scene_goal="主人公の弱点を冒頭で明示し共感獲得",
            emotional_goal="絶望→驚き→期待",
            character_arc=valid_arc,
            world_via_action="時間停止を体験シーンで提示",
            voice_constraints="「AはBである」形式禁止"
            # episode_number not set
        )

        result = validator.validate_for_step_11(intention)

        assert result.is_valid is True
        # Should warn about episode_number
        episode_warnings = [
            issue for issue in result.issues
            if issue.field_name == "episode_number"
        ]
        assert len(episode_warnings) > 0

    def test_validate_for_polish(self, validator, valid_intention):
        """Test validate_for_polish uses standard validation."""
        result = validator.validate_for_polish(valid_intention)

        assert result.is_valid is True
        assert result.checked_intention == valid_intention

    def test_character_arc_validation_all_fields_too_short(self, validator):
        """Test character arc validation catches all short fields."""
        # Create arc with all fields too short (will raise in __post_init__)
        with pytest.raises(ValueError):
            arc = CharacterArc(
                before_state="短",
                transition="短",
                after_state="短"
            )

    def test_character_arc_validation_meaningful_content(self, validator, valid_intention):
        """Test character arc validation for meaningful content."""
        # Valid intention should have meaningful content
        result = validator.validate(valid_intention)

        # Check no errors about meaningless content
        meaningless_errors = [
            issue for issue in result.issues
            if "meaningful content" in issue.message
        ]
        assert len(meaningless_errors) == 0



"""プロジェクト設定リポジトリ - インフラストラクチャ層

プロジェクト設定.yamlから設定情報を取得する
"""

from pathlib import Path

import yaml

from noveler.infrastructure.di.container import resolve_service
from noveler.infrastructure.factories.configuration_service_factory import get_configuration_manager


class YamlProjectSettingsRepository:
    """YAML形式のプロジェクト設定を扱うリポジトリ"""

    def get_genre(self, project_root: str | Path) -> str:
        """プロジェクトのジャンルを取得

        Args:
            project_root: プロジェクトのルートディレクトリ

        Returns:
            ジャンル文字列
        """
        try:
            path_service = resolve_service("IPathService")
            settings_file = path_service.get_project_config_file()
        except (ValueError, KeyError):
            # DIコンテナが設定されていない場合のフォールバック
            settings_file = self._resolve_project_config_path(project_root)

        if not settings_file.exists():
            # 企画書から取得を試みる
            return self._get_genre_from_proposal(project_root)

        try:
            with Path(settings_file).open(encoding="utf-8") as f:
                data = yaml.safe_load(f) or {}

            # ジャンル情報の取得(複数の可能性を考慮)
            genre = data.get("genre") or data.get("ジャンル") or data.get("category")

            if genre:
                return str(genre)
            # 企画書から取得を試みる
            return self._get_genre_from_proposal(project_root)

        except Exception:
            return "その他"

    def _get_genre_from_proposal(self, project_root: str | Path) -> str:
        """企画書.yamlからジャンルを取得"""
        try:
            path_service = resolve_service("IPathService")
            proposal_file = path_service.get_proposal_file()
        except (ValueError, KeyError):
            # DIコンテナが設定されていない場合のフォールバック
            proposal_file = Path(project_root) / "企画書.yaml"

        if not proposal_file.exists():
            return "その他"

        try:
            with Path(proposal_file).open(encoding="utf-8") as f:
                data = yaml.safe_load(f) or {}

            # 基本情報セクションから取得
            basic_info = data.get("基本情報", {})
            genre = basic_info.get("ジャンル") or basic_info.get("genre")

            if genre:
                return str(genre)

            # トップレベルからも探す
            genre = data.get("genre") or data.get("ジャンル")

            return str(genre) if genre else "その他"

        except Exception:
            return "その他"

    def _resolve_project_config_path(self, project_root: str | Path) -> Path:
        """Resolve the project config file path using templates with safe fallback."""
        filename = "プロジェクト設定.yaml"
        base = Path(project_root)

        # プロジェクト直下の .novelerrc.* を優先
        for rc_name in (".novelerrc.yaml", ".novelerrc.yml"):
            rc_path = base / rc_name
            if rc_path.exists():
                try:
                    from noveler.infrastructure.repositories.yaml_file_template_repository import (
                        YamlFileTemplateRepository,
                    )

                    repository = YamlFileTemplateRepository(config_path=rc_path)
                    candidate = repository.get_template("project_config")
                    if isinstance(candidate, str) and candidate.strip():
                        return base / candidate.strip()
                except Exception:
                    continue

        # グローバル設定（ConfigurationManager）を参照
        try:
            manager = get_configuration_manager()
            candidate = manager.get_file_template("project_config")
            if isinstance(candidate, str) and candidate.strip():
                filename = candidate.strip()
        except Exception:
            pass

        return base / filename

    def get_title(self, project_root: str | Path) -> str:
        """プロジェクトのタイトルを取得"""
        try:
            path_service = resolve_service("IPathService")
            settings_file = path_service.get_project_config_file()
        except (ValueError, KeyError):
            # DIコンテナが設定されていない場合のフォールバック
            settings_file = self._resolve_project_config_path(project_root)

        if not settings_file.exists():
            return Path(project_root).name  # ディレクトリ名を返す

        try:
            with Path(settings_file).open(encoding="utf-8") as f:
                data = yaml.safe_load(f) or {}

            title = data.get("title") or data.get("タイトル") or data.get("作品名")
            return str(title) if title else Path(project_root).name

        except Exception:
            return Path(project_root).name

    def get_target_readers(self, project_root: str | Path) -> str:
        """ターゲット読者層を取得"""
        # 読者分析.yamlから取得
        try:
            path_service = resolve_service("IPathService")
            reader_analysis_file = path_service.get_reader_analysis_file()
        except (ValueError, KeyError):
            # DIコンテナが設定されていない場合のフォールバック
            reader_analysis_file = Path(project_root) / "読者分析.yaml"

        if reader_analysis_file.exists():
            try:
                with Path(reader_analysis_file).open(encoding="utf-8") as f:
                    data = yaml.safe_load(f) or {}

                target = data.get("ターゲット読者層", {})
                if isinstance(target, dict):
                    age = target.get("年齢層", "")
                    gender = target.get("性別", "")
                    return f"{age} {gender}".strip()
                if isinstance(target, str):
                    return target

            except Exception as e:

                self.logger_service.warning("読者分析ファイル読み込みエラー: %s, エラー: %s", reader_analysis_file, e)

        # プロジェクト設定から取得
        try:
            path_service = resolve_service("IPathService")
            settings_file = path_service.get_project_config_file()
        except (ValueError, KeyError):
            settings_file = self._resolve_project_config_path(project_root)

        if settings_file.exists():
            try:
                with Path(settings_file).open(encoding="utf-8") as f:
                    data = yaml.safe_load(f) or {}

                target = data.get("target_readers") or data.get("ターゲット読者")
                return str(target) if target else "一般"

            except Exception as e:

                self.logger_service.warning("プロジェクト設定ファイル読み込みエラー: %s, エラー: %s", settings_file, e)

        return "一般"
